@@ -34,11 +34,11 @@
 **  ARRAY *ARRAY_Create(FILE *fp)   
 **    Creates a virtual array. Returns the handle of the array
 **
-**  ARRAY *ARRAY_Open(FILE *fp, unsigned long root_page) 
+**  ARRAY *ARRAY_Open(FILE *fp, sw_off_t root_page) 
 **    Opens an existent Virtual Array. root_page is de value returned by
 **    Array_Close. Returns de handle of the array.
 **
-**  unsigned long ARRAY_Close(ARRAY *arr)
+**  sw_off_t ARRAY_Close(ARRAY *arr)
 **    Closes and frees memory. arr is the value returned by ARRAY_Create or
 **    ARRAY_Open. Returns the root page of the array. This value must be
 **
@@ -64,26 +64,26 @@
 /* A ARRAY page size */
 #define ARRAY_PageSize 4096
 
-#define SizeInt32 4
+#define SizeOfElement sizeof(long)
 
 /* Round to ARRAY_PageSize */
-#define ARRAY_RoundPageSize(n) (((n) + ARRAY_PageSize - 1) & (~(ARRAY_PageSize - 1)))
+#define ARRAY_RoundPageSize(n) (((sw_off_t)(n) + (sw_off_t)(ARRAY_PageSize - 1)) & (~(sw_off_t)(ARRAY_PageSize - 1)))
 
-#define ARRAY_PageHeaderSize (1 * SizeInt32) 
+#define ARRAY_PageHeaderSize (1 * sizeof(sw_off_t)) 
 
 #define ARRAY_PageData(pg) ((pg)->data + ARRAY_PageHeaderSize)
-#define ARRAY_Data(pg,i) (ARRAY_PageData((pg)) + (i) * SizeInt32)
+#define ARRAY_Data(pg,i) (ARRAY_PageData((pg)) + (i) * SizeOfElement)
 
-#define ARRAY_SetNextPage(pg,num) ( *(int *)((pg)->data + 0 * SizeInt32) = PACKLONG(num))
+#define ARRAY_SetNextPage(pg,num) (sw_off_t)( *(sw_off_t *)((pg)->data + 0 * sizeof(sw_off_t)) = PACKFILEOFFSET(num))
 
-#define ARRAY_GetNextPage(pg,num) ( (num) = UNPACKLONG(*(int *)((pg)->data + 0 * SizeInt32)))
+#define ARRAY_GetNextPage(pg,num) ( (num) = UNPACKFILEOFFSET(*(sw_off_t *)((pg)->data + 0 * sizeof(sw_off_t))))
 
 
 int ARRAY_WritePageToDisk(FILE *fp, ARRAY_Page *pg)
 {
     ARRAY_SetNextPage(pg,pg->next);
-    fseek(fp,pg->page_number * ARRAY_PageSize,SEEK_SET);
-    return fwrite(pg->data,ARRAY_PageSize,1,fp);
+    sw_fseek(fp,(sw_off_t)((sw_off_t)pg->page_number * (sw_off_t)ARRAY_PageSize),SEEK_SET);
+    return sw_fwrite(pg->data,ARRAY_PageSize,1,fp);
 }
 
 int ARRAY_WritePage(ARRAY *b, ARRAY_Page *pg)
@@ -161,12 +161,12 @@ ARRAY_Page *tmp,*next;
     return 0;
 }
 
-ARRAY_Page *ARRAY_ReadPageFromDisk(FILE *fp, unsigned long page_number)
+ARRAY_Page *ARRAY_ReadPageFromDisk(FILE *fp, sw_off_t page_number)
 {
 ARRAY_Page *pg = (ARRAY_Page *)emalloc(sizeof(ARRAY_Page) + ARRAY_PageSize);
 
-    fseek(fp,page_number * ARRAY_PageSize,SEEK_SET);
-    fread(pg->data,ARRAY_PageSize, 1, fp);
+    sw_fseek(fp,(sw_off_t)(page_number * (sw_off_t)ARRAY_PageSize),SEEK_SET);
+    sw_fread(pg->data,ARRAY_PageSize, 1, fp);
 
     ARRAY_GetNextPage(pg,pg->next);
 
@@ -175,9 +175,9 @@ ARRAY_Page *pg = (ARRAY_Page *)emalloc(sizeof(ARRAY_Page) + ARRAY_PageSize);
     return pg;
 }
 
-ARRAY_Page *ARRAY_ReadPage(ARRAY *b, unsigned long page_number)
+ARRAY_Page *ARRAY_ReadPage(ARRAY *b, sw_off_t page_number)
 {
-int hash = page_number % ARRAY_CACHE_SIZE;
+int hash = (int)(page_number % (sw_off_t)ARRAY_CACHE_SIZE);
 ARRAY_Page *tmp;
     if((tmp = b->cache[hash]))
     {
@@ -202,33 +202,33 @@ ARRAY_Page *tmp;
 ARRAY_Page *ARRAY_NewPage(ARRAY *b)
 {
 ARRAY_Page *pg;
-long offset;
+sw_off_t offset;
 FILE *fp = b->fp;
 int hash;
 int size = ARRAY_PageSize;
 
     /* Get file pointer */
-    if(fseek(fp,0,SEEK_END) !=0)
+    if(sw_fseek(fp,(sw_off_t)0,SEEK_END) !=0)
         progerrno("Failed to seek to eof: ");
 
-    offset = ftell(fp);
+    offset = sw_ftell(fp);
     /* Round up file pointer */
     offset = ARRAY_RoundPageSize(offset);
 
     /* Set new file pointer - data will be aligned */
-    if(fseek(fp,offset, SEEK_SET)!=0 || offset != ftell(fp))
+    if(sw_fseek(fp,offset, SEEK_SET)!=0 || offset != sw_ftell(fp))
         progerrno("Failed during seek: ");
 
 
     pg = (ARRAY_Page *)emalloc(sizeof(ARRAY_Page) + size);
     memset(pg,0,sizeof(ARRAY_Page) + size);
     /* Reserve space in file */
-    if(fwrite(pg->data,1,size,fp)!=size || ((long)size + offset) != ftell(fp))
+    if(sw_fwrite(pg->data,1,size,fp)!=size || ((sw_off_t)size + offset) != sw_ftell(fp))
         progerrno("Failed to write ARRAY_page: ");
 
     pg->next = 0;
 
-    pg->page_number = offset/ARRAY_PageSize;
+    pg->page_number = offset / (sw_off_t)ARRAY_PageSize;
 
     /* add to cache */
     pg->modified = 1;
@@ -298,7 +298,7 @@ int size = ARRAY_PageSize;
 }
 
 
-ARRAY *ARRAY_Open(FILE *fp, unsigned long root_page)
+ARRAY *ARRAY_Open(FILE *fp, sw_off_t root_page)
 {
 ARRAY *b;
 int size = ARRAY_PageSize;
@@ -310,9 +310,9 @@ int size = ARRAY_PageSize;
     return b;
 }
 
-unsigned long ARRAY_Close(ARRAY *bt)
+sw_off_t ARRAY_Close(ARRAY *bt)
 {
-unsigned long root_page = bt->root_page;
+sw_off_t root_page = bt->root_page;
     ARRAY_FlushCache(bt);
     ARRAY_CleanCache(bt);
     efree(bt);
@@ -322,17 +322,17 @@ unsigned long root_page = bt->root_page;
 
 int ARRAY_Put(ARRAY *b, int index, unsigned long value)
 {
-unsigned long next_page; 
+sw_off_t next_page; 
 ARRAY_Page *root_page, *tmp = NULL, *prev; 
 int i, hash, page_reads, page_index;
 
-    page_reads = index / ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    hash = page_reads % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    page_reads /= ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    page_index = index % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
+    page_reads = index / ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    hash = page_reads % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    page_reads /= ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    page_index = index % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
 
     root_page = ARRAY_ReadPage(b, b->root_page);
-    next_page = UNPACKLONG(*(unsigned long *)ARRAY_Data(root_page, hash));
+    next_page = UNPACKFILEOFFSET(*(sw_off_t *)ARRAY_Data(root_page, hash));
 
     prev = NULL;
     for(i = 0; i <= page_reads; i++)
@@ -343,7 +343,7 @@ int i, hash, page_reads, page_index;
             ARRAY_WritePage(b,tmp);
             if(!i)
             {
-                *(unsigned long *)ARRAY_Data(root_page,hash) = PACKLONG(tmp->page_number);
+                *(sw_off_t *)ARRAY_Data(root_page,hash) = PACKFILEOFFSET(tmp->page_number);
                  ARRAY_WritePage(b,root_page);
             }
             else
@@ -372,17 +372,19 @@ int i, hash, page_reads, page_index;
 
 unsigned long ARRAY_Get(ARRAY *b, int index)
 {
-unsigned long next_page, value; 
+sw_off_t next_page;
+unsigned long value; 
 ARRAY_Page *root_page, *tmp;
 int i, hash, page_reads, page_index;
 
-    page_reads = index / ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    hash = page_reads % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    page_reads /= ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
-    page_index = index % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeInt32);
+    page_reads = index / ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    hash = page_reads % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    page_reads /= ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
+    page_index = index % ((ARRAY_PageSize - ARRAY_PageHeaderSize) / SizeOfElement);
 
     root_page = ARRAY_ReadPage(b, b->root_page);
-    next_page = UNPACKLONG(*(unsigned long *)ARRAY_Data(root_page, hash));
+/* $$$$ to be fixed $$$ */
+    next_page = UNPACKLONG(*(long *)ARRAY_Data(root_page, hash));
 
     tmp = NULL;
     for(i = 0; i <= page_reads; i++)
@@ -434,11 +436,11 @@ unsigned long root_page;
 
 
 
-    fp = fopen("kkkkk",F_WRITE_BINARY);
-    fclose(fp);
-    fp = fopen("kkkkk",F_READWRITE_BINARY);
+    fp = sw_fopen("kkkkk",F_WRITE_BINARY);
+    sw_fclose(fp);
+    fp = sw_fopen("kkkkk",F_READWRITE_BINARY);
 
-    fwrite("aaa",1,3,fp);
+    sw_fwrite("aaa",1,3,fp);
 
 printf("\n\nIndexing\n\n");
 
@@ -458,12 +460,12 @@ printf("\n\nIndexing\n\n");
     }
 
     root_page = ARRAY_Close(bt);
-    fclose(fp);
+    sw_fclose(fp);
 
 printf("\n\nUnfreed %d\n\n",num);
 printf("\n\nSearching\n\n");
 
-    fp = fopen("kkkkk",F_READ_BINARY);
+    fp = sw_fopen("kkkkk",F_READ_BINARY);
     bt = ARRAY_Open(fp, root_page);
 
     for(i=0;i<N_TEST;i++)
@@ -476,7 +478,7 @@ printf("\n\nSearching\n\n");
 
     root_page = ARRAY_Close(bt);
 
-    fclose(fp);
+    sw_fclose(fp);
 printf("\n\nUnfreed %d\n\n",num);
 
 }
