@@ -3,14 +3,14 @@
 
 typedef struct BTREE_Page
 {
-    unsigned long      next;    /* Next Page */
-    unsigned long      prev;    /* Previous Page */
+    sw_off_t           next;    /* Next Page */
+    sw_off_t           prev;    /* Previous Page */
     unsigned int       size;    /* Size of page */
     unsigned int       n;       /* Number of keys in page */
     unsigned int       flags;
     unsigned int       data_end;
 
-    unsigned long      page_number;
+    sw_off_t           page_number;
     int                modified;
     int                in_use;
 
@@ -23,27 +23,23 @@ typedef struct BTREE_Page
 
 typedef struct BTREE
 {
-    unsigned long root_page;
+    sw_off_t root_page;
     int page_size;
     struct BTREE_Page *cache[BTREE_CACHE_SIZE];
     int levels;
-    unsigned long tree[1024];
+    sw_off_t tree[1024];
           /* Values for sequential reading */
-    unsigned long current_page;
+    sw_off_t current_page;
     unsigned long current_position;
 
     FILE *fp;
 } BTREE;
 
 BTREE *BTREE_Create(FILE *fp, unsigned int size);
-BTREE *BTREE_Open(FILE *fp, int size, unsigned long root_page);
-unsigned long BTREE_Close(BTREE *bt);
-int BTREE_Insert(BTREE *b, unsigned char *key, int key_len, unsigned long data_pointer);
-long BTREE_Search(BTREE *b, unsigned char *key, int key_len, unsigned char **found, int *found_len, int exact_match);
-long BTREE_Next(BTREE *b, unsigned char **found, int *found_len);
-int BTREE_Update(BTREE *b, unsigned char *key, int key_len, unsigned long new_data_pointer);
-
-
-
-
+BTREE *BTREE_Open(FILE *fp, int size, sw_off_t root_page);
+sw_off_t BTREE_Close(BTREE *bt);
+int BTREE_Insert(BTREE *b, unsigned char *key, int key_len, sw_off_t data_pointer);
+sw_off_t BTREE_Search(BTREE *b, unsigned char *key, int key_len, unsigned char **found, int *found_len, int exact_match);
+sw_off_t BTREE_Next(BTREE *b, unsigned char **found, int *found_len);
+int BTREE_Update(BTREE *b, unsigned char *key, int key_len, sw_off_t new_data_pointer);
 
