@@ -17,14 +17,7 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-/*
-** use _AP() for easier cross-compiler (non-ANSI) porting 
-** <return value> <functionname> _AP( (<arg prototypes>) );
-*/
 
-int isokword _AP ((char *));
-/*
-int isvowel _AP ((char)); 
-*/
-int hasokchars _AP ((char *));
-char *hasfilter _AP((char *filename, struct filter *filterlist));
+int isokword (SWISH *,char *,IndexFILE *);
+int getdoctype (char *filename, struct IndexContents *indexcontents);
+struct StoreDescription *hasdescription (int, struct StoreDescription *);
