@@ -61,20 +61,22 @@ static int    write_hash_words_to_header(SWISH *sw, int header_ID, struct swline
         itmp[1] = PACKLONG(itmp[1]); \
         DB_WriteHeaderData((sw),(id), (unsigned char *)itmp, sizeof(long) * 2, (DB)); \
 }
-#define write_header_int3(sw,id,num1,num2,num3,DB)\
+#define write_header_int4(sw,id,num1,num2,num3,num4,DB)\
 {  \
-        unsigned long itmp[3]; \
+        unsigned long itmp[4]; \
         itmp[0] = (num1); \
         itmp[1] = (num2); \
         itmp[2] = (num3); \
+        itmp[3] = (num4); \
         itmp[0] = PACKLONG(itmp[0]); \
         itmp[1] = PACKLONG(itmp[1]); \
         itmp[2] = PACKLONG(itmp[2]); \
-        DB_WriteHeaderData((sw),(id), (unsigned char *)itmp, sizeof(long) * 3, (DB)); \
+        itmp[3] = PACKLONG(itmp[3]); \
+        DB_WriteHeaderData((sw),(id), (unsigned char *)itmp, sizeof(long) * 4, (DB)); \
 }
 
 
-void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filename, int totalwords, int totalfiles, int removedfiles, int merged)
+void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filename, int totalwords, int totalfiles, int total_word_positions, int removedfiles, int merged)
 {
     char   *c,
            *tmp;
@@ -92,9 +94,9 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
     write_header_int(sw, MERGED_ID, merged, DB);
     DB_WriteHeaderData(sw, NAMEHEADER_ID, (unsigned char *)header->indexn, strlen(header->indexn) + 1, DB);
     DB_WriteHeaderData(sw, SAVEDASHEADER_ID, (unsigned char *)c, strlen(c) + 1, DB);
-    write_header_int3(sw, COUNTSHEADER_ID, totalwords, totalfiles, removedfiles, DB);
-    tmp = getTheDateISO(); 
-    DB_WriteHeaderData(sw, INDEXEDONHEADER_ID, (unsigned char *)tmp, strlen(tmp) + 1,DB); 
+    write_header_int4(sw, COUNTSHEADER_ID, totalwords, totalfiles, total_word_positions, removedfiles, DB);
+    tmp = getTheDateISO();
+    DB_WriteHeaderData(sw, INDEXEDONHEADER_ID, (unsigned char *)tmp, strlen(tmp) + 1,DB);
     efree(tmp);
     DB_WriteHeaderData(sw, DESCRIPTIONHEADER_ID, (unsigned char *)header->indexd, strlen(header->indexd) + 1, DB);
     DB_WriteHeaderData(sw, POINTERHEADER_ID, (unsigned char *)header->indexp, strlen(header->indexp) + 1, DB);
@@ -111,7 +113,7 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
     DB_WriteHeaderData(sw, ENDCHARSHEADER_ID, (unsigned char *)header->endchars, strlen(header->endchars) + 1, DB);
     DB_WriteHeaderData(sw, IGNOREFIRSTCHARHEADER_ID, (unsigned char *)header->ignorefirstchar, strlen(header->ignorefirstchar) + 1, DB);
     DB_WriteHeaderData(sw, IGNORELASTCHARHEADER_ID, (unsigned char *)header->ignorelastchar, strlen(header->ignorelastchar) + 1,DB);
-    /* Removed - Patents 
+    /* Removed - Patents
     write_header_int(FILEINFOCOMPRESSION_ID, header->applyFileInfoCompression, DB);
     */
 
@@ -119,7 +121,7 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
 
     /* Jose Ruiz 06/00 Added this line to delimite the header */
     write_integer_table_to_header(sw, TRANSLATECHARTABLE_ID, header->translatecharslookuptable, sizeof(header->translatecharslookuptable) / sizeof(int), DB);
-    
+
     /* Other header stuff */
 
     /* StopWords */
@@ -147,7 +149,7 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
 */
 void    write_word(SWISH * sw, ENTRY * ep, IndexFILE * indexf)
 {
-    long    wordID;
+    sw_off_t    wordID;
 
     wordID = DB_GetWordID(sw, indexf->DB);
 
@@ -163,7 +165,7 @@ void    write_word(SWISH * sw, ENTRY * ep, IndexFILE * indexf)
 */
 void    update_wordID(SWISH * sw, ENTRY * ep, IndexFILE * indexf)
 {
-    long    wordID;
+    sw_off_t    wordID;
 
     wordID = DB_GetWordID(sw, indexf->DB);
 
@@ -172,7 +174,7 @@ void    update_wordID(SWISH * sw, ENTRY * ep, IndexFILE * indexf)
     ep->u1.wordID = wordID;
 }
 
-void    delete_worddata(SWISH * sw, long wordID, IndexFILE * indexf)
+void    delete_worddata(SWISH * sw, sw_off_t wordID, IndexFILE * indexf)
 {
     DB_DeleteWordData(sw,wordID,indexf->DB);
 }
@@ -219,9 +221,9 @@ void build_worddata(SWISH * sw, ENTRY * ep)
         memcpy((char *)&chunk_size,(char *)p,sizeof(chunk_size));
         p += sizeof(chunk_size);
 
-        if(curmetaID!=metaID) 
+        if(curmetaID!=metaID)
         {
-            if(curmetaID) 
+            if(curmetaID)
             {
                 /* Write in previous meta (curmetaID)
                 ** file offset to next meta */
@@ -229,7 +231,7 @@ void build_worddata(SWISH * sw, ENTRY * ep)
                 PACKLONG2(tmp,sw->Index->worddata_buffer+curmetanamepos);
             }
                 /* Check for enough memory */
-                /* 
+                /*
                 ** MAXINTCOMPSIZE is for the worst case metaID
                 **
                 ** sizeof(long) is to leave four bytes to
@@ -255,7 +257,7 @@ void build_worddata(SWISH * sw, ENTRY * ep)
 
             /* preserve position for offset to next
             ** metaname. We do not know its size
-            ** so store it as a packed long */ 
+            ** so store it as a packed long */
             curmetanamepos=q - sw->Index->worddata_buffer;
 
             /* Store 0 and increase pointer */
@@ -267,7 +269,7 @@ void build_worddata(SWISH * sw, ENTRY * ep)
 
 
         /* Store all data for this chunk */
-        /* First check for enough space 
+        /* First check for enough space
         **
         ** 1 is for the trailing '\0'
         */
@@ -298,7 +300,7 @@ void build_worddata(SWISH * sw, ENTRY * ep)
 
     sz_worddata = q - sw->Index->worddata_buffer;
 
-    /* Adjust word positions. 
+    /* Adjust word positions.
     ** if ignorelimit was set and some new stopwords weee found, positions
     ** are recalculated
     ** Also call it even if we have not set IgnoreLimit to calesce word chunks
@@ -322,7 +324,12 @@ void write_worddata(SWISH * sw, ENTRY * ep, IndexFILE * indexf )
 
     /* Get some extra compression */
     remove_worddata_longs(sw->Index->worddata_buffer,&sw->Index->sz_worddata_buffer);
-    zlib_size = compress_worddata(sw->Index->worddata_buffer, sw->Index->sz_worddata_buffer);
+
+    if(sw->compressPositions)
+        zlib_size = compress_worddata(sw->Index->worddata_buffer, sw->Index->sz_worddata_buffer,sw->Index->swap_locdata);
+    else
+        zlib_size = sw->Index->sz_worddata_buffer;
+
     /* Write worddata */
     DB_WriteWordData(sw, ep->u1.wordID,sw->Index->worddata_buffer,zlib_size, sw->Index->sz_worddata_buffer - zlib_size ,indexf->DB);
 
@@ -343,9 +350,10 @@ int tfreq1, tfreq2;
 unsigned char *p1, *p2, *p;
 int curmetaID_1,curmetaID_2,metadata_length_1,num_metaids1;
 unsigned long nextposmetaname_1,nextposmetaname_2, curmetanamepos, curmetanamepos_1, curmetanamepos_2, tmp;
-int last_filenum, filenum, tmpval, frequency, *posdata;
+int last_filenum, filenum, tmpval, frequency;
+unsigned int *posdata;
 #define POSDATA_STACK 2000
-int stack_posdata[POSDATA_STACK];  /* Just to avoid the overhead of malloc/free */
+unsigned int stack_posdata[POSDATA_STACK];  /* Just to avoid the overhead of malloc/free */
 unsigned char r_flag, *w_flag;
 unsigned char *q;
 
@@ -368,7 +376,7 @@ unsigned char *q;
     if(maxtotsize > sw->Index->len_worddata_buffer)
     {
          sw->Index->len_worddata_buffer = maxtotsize + 2000;
-         sw->Index->worddata_buffer = (unsigned char *) erealloc(sw->Index->worddata_buffer,sw->Index->len_worddata_buffer); 
+         sw->Index->worddata_buffer = (unsigned char *) erealloc(sw->Index->worddata_buffer,sw->Index->len_worddata_buffer);
     }
     /* Preserve new data in a local copy - sw->Index->worddata_buffer is the final destination
     ** of data
@@ -382,7 +390,7 @@ unsigned char *q;
 
     /* Set pointers to all buffers */
     p1 = olddata;
-    p2 = newdata; 
+    p2 = newdata;
     q = p = sw->Index->worddata_buffer;
 
     /* Now read tfrequency */
@@ -394,19 +402,19 @@ unsigned char *q;
     /* Now look for MetaIDs */
     curmetaID_1 = uncompress2(&p1);
     curmetaID_2 = uncompress2(&p2);
-   
-    /* Old data is compressed in a different more optimized schema */ 
-    metadata_length_1 = uncompress2(&p1); 
+
+    /* Old data is compressed in a different more optimized schema */
+    metadata_length_1 = uncompress2(&p1);
     nextposmetaname_1 = p1 - olddata + metadata_length_1;
-    
+
     curmetanamepos_1 = p1 - olddata;
-    nextposmetaname_2 = UNPACKLONG2(p2); 
+    nextposmetaname_2 = UNPACKLONG2(p2);
     p2 += sizeof(long);
 
     curmetanamepos_2 = p2 - newdata;
 
     while(curmetaID_1 && curmetaID_2)
-    {            
+    {
         p = compress3(min(curmetaID_1,curmetaID_2),p);
 
         curmetanamepos = p - sw->Index->worddata_buffer;
@@ -426,13 +434,13 @@ unsigned char *q;
             {
                 /* Read on all items */
                 uncompress_location_values(&p1,&r_flag,&tmpval,&frequency);
-                last_filenum += tmpval;  
+                last_filenum += tmpval;
 
                 if(frequency > POSDATA_STACK)
-                    posdata = (int *) emalloc(frequency * sizeof(int));
+                    posdata = (unsigned int *) emalloc(frequency * sizeof(int));
                 else
                     posdata = stack_posdata;
-    
+
                 /* Read and discard positions just to advance pointer */
                 uncompress_location_positions(&p1,r_flag,frequency,posdata);
                 if(posdata!=stack_posdata)
@@ -455,7 +463,7 @@ unsigned char *q;
             if(curmetaID_1)
             {
                 curmetaID_1 = uncompress2(&p1);  /* Next metaID */
-                metadata_length_1 = uncompress2(&p1); 
+                metadata_length_1 = uncompress2(&p1);
                 nextposmetaname_1 = p1 - olddata + metadata_length_1;
                 curmetanamepos_1 = p1 - olddata;
             }
@@ -466,7 +474,7 @@ unsigned char *q;
             uncompress_location_values(&p2,&r_flag,&tmpval,&frequency);
             filenum = tmpval;  /* First filenum in chunk */
             if(frequency > POSDATA_STACK)
-                posdata = (int *) emalloc(frequency * sizeof(int));
+                posdata = (unsigned int *) emalloc(frequency * sizeof(int));
             else
                 posdata = stack_posdata;
 
@@ -492,7 +500,7 @@ unsigned char *q;
             if(curmetaID_2)
             {
                 curmetaID_2 = uncompress2(&p2);  /* Next metaID */
-                nextposmetaname_2 = UNPACKLONG2(p2); 
+                nextposmetaname_2 = UNPACKLONG2(p2);
                 p2 += sizeof(long);
                 curmetanamepos_2 = p2 - newdata;
             }
@@ -509,7 +517,7 @@ unsigned char *q;
             else
             {
                 curmetaID_1 = uncompress2(&p1);  /* Next metaID */
-                metadata_length_1 = uncompress2(&p1); 
+                metadata_length_1 = uncompress2(&p1);
                 nextposmetaname_1 = p1 - olddata + metadata_length_1;
                 curmetanamepos_1 = p1 - olddata;
             }
@@ -526,16 +534,16 @@ unsigned char *q;
             else
             {
                 curmetaID_2 = uncompress2(&p2);  /* Next metaID */
-                nextposmetaname_2 = UNPACKLONG2(p2); 
+                nextposmetaname_2 = UNPACKLONG2(p2);
                 p2 += sizeof(long);
                 curmetanamepos_2 = p2 - newdata;
             }
         }
         /* Put nextmetaname offset */
         PACKLONG2(p - sw->Index->worddata_buffer, sw->Index->worddata_buffer + curmetanamepos);
-       
+
     } /* while */
-  
+
     /* Add the rest of the data if exists */
     while(curmetaID_1)
     {
@@ -557,7 +565,7 @@ unsigned char *q;
         else
         {
             curmetaID_1 = uncompress2(&p1);  /* Next metaID */
-            metadata_length_1 = uncompress2(&p1); 
+            metadata_length_1 = uncompress2(&p1);
             nextposmetaname_1 = p1 - olddata + metadata_length_1;
             curmetanamepos_1 = p1 - olddata;
         }
@@ -585,7 +593,7 @@ unsigned char *q;
         else
         {
             curmetaID_2 = uncompress2(&p2);  /* Next metaID */
-            nextposmetaname_2 = UNPACKLONG2(p2); 
+            nextposmetaname_2 = UNPACKLONG2(p2);
             p2+= sizeof(long);
             curmetanamepos_2= p2 - newdata;
         }
@@ -600,6 +608,7 @@ unsigned char *q;
 }
 
 /* Writes the list of metaNames into the DB index
+ *  (should maybe be in metanames.c)
 */
 
 void    write_MetaNames(SWISH *sw, int id, INDEXDATAHEADER * header, void *DB)
@@ -621,7 +630,7 @@ void    write_MetaNames(SWISH *sw, int id, INDEXDATAHEADER * header, void *DB)
 
     fields = 5;  // len, metaID, metaType, alias, rank_bias
 
-     
+
     /* Compute buffer size */
     for (sz_buffer = 0 , i = 0; i < header->metaCounter; i++)
     {
@@ -629,7 +638,7 @@ void    write_MetaNames(SWISH *sw, int id, INDEXDATAHEADER * header, void *DB)
         len = strlen(entry->metaName);
         sz_buffer += len + fields * MAXINTCOMPSIZE; /* compress can use MAXINTCOMPSIZE bytes in worse case,  */
     }
-    
+
     sz_buffer += MAXINTCOMPSIZE;  /* Add extra MAXINTCOMPSIZE for the number of metanames */
 
     s = buffer = (unsigned char *) emalloc(sz_buffer);
@@ -646,7 +655,7 @@ void    write_MetaNames(SWISH *sw, int id, INDEXDATAHEADER * header, void *DB)
         s = compress3(entry->metaID, s);
         s = compress3(entry->metaType, s);
         s = compress3(entry->alias+1, s);  /* keep zeros away from compress3, I believe */
-        s = compress3(entry->sort_len, s); 
+        s = compress3(entry->sort_len, s);
         s = compress3(entry->rank_bias+RANK_BIAS_RANGE+1, s);
     }
     DB_WriteHeaderData(sw, id,buffer,s-buffer,DB);
@@ -666,7 +675,7 @@ static int    write_hash_words_to_header(SWISH *sw, int header_ID, struct swline
             sz_buffer;
     char   *buffer, *s;
     struct swline *sp = NULL;
-        
+
     /* Let's count the words */
 
     if ( !hash )
@@ -717,7 +726,7 @@ int write_integer_table_to_header(SWISH *sw, int id, int table[], int table_size
             tmp;
     char   *s;
     char   *buffer;
-    
+
     s = buffer = (char *) emalloc((table_size + 1) * MAXINTCOMPSIZE);
 
     s = (char *)compress3(table_size,(unsigned char *)s);   /* Put the number of elements */
@@ -741,7 +750,6 @@ void setTotalWordsPerFile(IndexFILE *indexf, int idx,int wordcount)
 {
 #ifdef USE_BTREE
         DB_WriteTotalWordsPerFile(indexf->sw, idx, wordcount, indexf->DB);
-
 #else
 INDEXDATAHEADER *header = &indexf->header;
 
@@ -801,35 +809,35 @@ int     DB_InitWriteWords(SWISH *sw, void *DB)
    return sw->Db->DB_InitWriteWords(DB);
 }
 
-long    DB_GetWordID(SWISH *sw, void *DB)
+sw_off_t    DB_GetWordID(SWISH *sw, void *DB)
 {
    return sw->Db->DB_GetWordID(DB);
 }
 
-int     DB_WriteWord(SWISH *sw, char *word, long wordID, void *DB)
+int     DB_WriteWord(SWISH *sw, char *word, sw_off_t wordID, void *DB)
 {
    return sw->Db->DB_WriteWord(word, wordID, DB);
 }
 
 #ifdef USE_BTREE
-int     DB_UpdateWordID(SWISH *sw, char *word, long wordID, void *DB)
+int     DB_UpdateWordID(SWISH *sw, char *word, sw_off_t wordID, void *DB)
 {
    return sw->Db->DB_UpdateWordID(word, wordID, DB);
 }
 
-int     DB_DeleteWordData(SWISH *sw, long wordID, void *DB)
+int     DB_DeleteWordData(SWISH *sw, sw_off_t wordID, void *DB)
 {
    return sw->Db->DB_DeleteWordData(wordID, DB);
 }
 
 #endif
 
-int     DB_WriteWordHash(SWISH *sw, char *word, long wordID, void *DB)
+int     DB_WriteWordHash(SWISH *sw, char *word, sw_off_t wordID, void *DB)
 {
    return sw->Db->DB_WriteWordHash(word, wordID, DB);
 }
 
-long    DB_WriteWordData(SWISH *sw, long wordID, unsigned char *worddata, int data_size, int saved_bytes, void *DB)
+long    DB_WriteWordData(SWISH *sw, sw_off_t wordID, unsigned char *worddata, int data_size, int saved_bytes, void *DB)
 {
    return sw->Db->DB_WriteWordData(wordID, worddata, data_size, saved_bytes, DB);
 }
