@@ -1,5 +1,5 @@
 /*
-$Id: docprop.c,v 1.117.2.1 2004/01/24 18:32:30 whmoseley Exp $
+$Id: docprop.c,v 1.119 2004/04/28 20:17:27 whmoseley Exp $
 **
 ** Functions to manage the index's Document Properties 
 **
@@ -428,6 +428,8 @@ unsigned long SwishResultPropertyULong(RESULT *result, char *pname)
                     "Property '%s' is not numeric", pname );
         value = ULONG_MAX;
     } 
+    else 
+        value = pv->value.v_ulong;
 
     freeResultPropValue( pv );
 
