@@ -1,5 +1,5 @@
 /*
-$Id: index.c,v 1.222.2.4 2004/01/30 19:47:17 whmoseley Exp $
+$Id: index.c,v 1.232 2004/12/06 22:08:41 whmoseley Exp $
 **
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
@@ -193,13 +193,13 @@ void initModule_Index (SWISH  *sw)
     idx->IgnoreLimitPositionsArray = NULL;
 
        /* Swapping access file functions */
-    idx->swap_tell = ftell;
-    idx->swap_write = fwrite;
-    idx->swap_close = fclose;
-    idx->swap_seek = fseek;
-    idx->swap_read = fread;
-    idx->swap_getc = fgetc;
-    idx->swap_putc = fputc;
+    idx->swap_tell = sw_ftell;
+    idx->swap_write = sw_fwrite;
+    idx->swap_close = sw_fclose;
+    idx->swap_seek = sw_fseek;
+    idx->swap_read = sw_fread;
+    idx->swap_getc = sw_fgetc;
+    idx->swap_putc = sw_fputc;
 
     for( i = 0; i <MAX_LOC_SWAP_FILES ; i++)
     {
@@ -232,7 +232,7 @@ void initModule_Index (SWISH  *sw)
     idx->tmpdir = estrdup(".");
 
     /* By default, we are not in update mode */
-    idx->update_mode = 0;
+    idx->update_mode = MODE_INDEX;
 
     return;
 }
@@ -719,6 +719,9 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
     char        strType[30];
     int         i;
     FileRec     fi;  /* place to hold doc properties */
+#ifdef USE_BTREE
+    int old_filenum;
+#endif
 
     memset( &fi, 0, sizeof( FileRec ) );
 
@@ -763,7 +766,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
 
     else if ( !fprop->fp )
     {
-        fprop->fp = fopen(fprop->work_path, F_READ_TEXT );
+        fprop->fp = sw_fopen(fprop->work_path, F_READ_TEXT );
 
         if ( !fprop->fp )
         {
@@ -787,9 +790,11 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
 
 #ifdef USE_BTREE
     /** Let's see if file already exits when in update mode */
-    if(sw->Index->update_mode)
+    switch(sw->Index->update_mode)
     {
-        int old_filenum;
+
+      case MODE_UPDATE:      /* Update mode */
+
         DB_ReadFileNum(sw,&old_filenum,fprop->real_path,strlen(fprop->real_path),indexf->DB);
         /* If exits a previous file with the same real_path ... */
         if(old_filenum)
@@ -808,8 +813,8 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
             memset(&fi, 0, sizeof( FileRec ));
             fi.filenum = old_filenum;
             cp = ReadSingleDocPropertiesFromDisk(cur_index, &fi, cur_index->modified_meta->metaID, 0 );
- 
-            /* Crate a property based on mtime in order to use it
+
+            /* Create a property based on mtime in order to use it
             ** for comparing properties in Compare_Property routine */
             tmp = PACKLONG(fprop->mtime);
             wp = CreateProperty( cur_index->modified_meta, (unsigned char *)&tmp, sizeof( tmp ), 1, &error_flag );
@@ -835,12 +840,29 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
                 cur_index->header.removedfiles++;
             }
         }
+        break;
+
+
+
+      case MODE_REMOVE:      /* Remove mode */
+
+        DB_ReadFileNum(sw,&old_filenum,fprop->real_path,strlen(fprop->real_path),indexf->DB);
+        /* If exits a previous file with the same real_path remove it */
+        if(old_filenum)
+        {
+            IndexFILE   *cur_index = sw->indexlist;
+            /* Remove old filenum and continue */
+            DB_RemoveFileNum(sw,old_filenum,indexf->DB);
+            cur_index->header.removedfiles++;
+        }
+        return;
+        break;
     }
 
 #endif
 
     /** Read the buffer, if not a stream parser **/
-    
+
 #ifdef HAVE_LIBXML2
     if ( !fprop->doctype || fprop->doctype == HTML2 || fprop->doctype == XML2 || fprop->doctype == TXT2 )
         rd_buffer = NULL;
@@ -947,7 +969,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
         if (fprop->hasfilter)
             FilterClose(fprop->fp); /* close filter pipe - should the filter be flushed? */
         else
-            fclose(fprop->fp); /* close file */
+            sw_fclose(fprop->fp); /* close file */
     }
     /* Else, it's -S prog so make sure we read all the bytes we are suppose to read! */
     /* Can remove the check for fprop->bytes_read once read_stream is no longer used */
@@ -967,7 +989,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
         else if (wordcount == -2)
             printf(" (Skipped due to 'FileRules title' setting)\n");
         else if (wordcount == -3)
-            printf(" (Skipped due to Robots Excluion Rule in meta tag)\n");
+            printf(" (Skipped due to Robots Exclusion Rule in meta tag)\n");
         fflush(stdout);
     }
 
@@ -992,8 +1014,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
     /* write properties to disk, and release docprop array (and the prop index array) */
     /* Currently this just passes sw, and assumes only one index file when indexing */
     WritePropertiesToDisk( sw , &fi );
-
-
+	
 #ifdef USE_BTREE
     /* Add the value pair (real_path, filenum) to the database */
     DB_WriteFileNum(sw,fi.filenum,fprop->real_path,strlen(fprop->real_path),indexf->DB);
@@ -1436,8 +1457,8 @@ void getPositionsFromIgnoreLimitWords(SWISH * sw)
             frequency,
             tmpval,
             filenum;
-    int    *positions;
-    int     local_positions[MAX_STACK_POSITIONS];
+    unsigned int    *posdata;
+    unsigned int     local_posdata[MAX_STACK_POSITIONS];
 
     LOCATION *l, *next;
     ENTRY  *ep,
@@ -1574,11 +1595,11 @@ void getPositionsFromIgnoreLimitWords(SWISH * sw)
                      filenum += tmpval;
 
                      if(frequency > MAX_STACK_POSITIONS)
-                         positions = (int *) emalloc(frequency * sizeof(int));
+                         posdata = (unsigned int *) emalloc(frequency * sizeof(int));
                      else
-                         positions = local_positions;
+                         posdata = local_posdata;
 
-                     uncompress_location_positions(&p,flag,frequency,positions);
+                     uncompress_location_positions(&p,flag,frequency,posdata);
 
                      chunk_size -= (p-q);
          
@@ -1600,13 +1621,13 @@ void getPositionsFromIgnoreLimitWords(SWISH * sw)
                      for (m = fpos->n * 2, k = 0; k < frequency; k++)
                      {
                          fpos->pos[m++] = metaID;
-                         fpos->pos[m++] = GET_POSITION(positions[k]);
+                         fpos->pos[m++] = GET_POSITION(posdata[k]);
                      }
 
                      fpos->n += frequency;
 
-                     if(positions != local_positions)
-                         efree(positions);
+                     if(posdata != local_posdata)
+                         efree(posdata);
                 }
                 l = next;
             }
@@ -1640,15 +1661,19 @@ void adjustWordPositions(unsigned char *worddata, int *sz_worddata, int n_files,
             metaID,
             tmpval,
             r_filenum, 
-            w_filenum,
-           *posdata;
+            w_filenum;
+    unsigned int       *posdata;
     int     i,j,k;
     unsigned long    r_nextposmeta;
     unsigned char   *w_nextposmeta;
-    int     local_posdata[MAX_STACK_POSITIONS];
+    unsigned int     local_posdata[MAX_STACK_POSITIONS];
     unsigned char r_flag, *w_flag;
     unsigned char *p, *q;
 
+	/* TODO
+	total_word_pos count for index header -- should be adjusted to reflect any positions +/- */
+	
+
     p = worddata;
 
     tmpval = uncompress2(&p);     /* tfrequency */
@@ -1667,7 +1692,7 @@ void adjustWordPositions(unsigned char *worddata, int *sz_worddata, int n_files,
         if(frequency <= MAX_STACK_POSITIONS)
             posdata = local_posdata;
         else
-            posdata = (int *) emalloc(frequency * sizeof(int));
+            posdata = (unsigned int *) emalloc(frequency * sizeof(int));
 
         uncompress_location_positions(&p,r_flag,frequency,posdata);
 
@@ -1964,7 +1989,7 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
             write_word(sw, epi, indexf);
         }
         else
-            epi->u1.wordID = -1;  /* flag as a stop word */
+            epi->u1.wordID = (sw_off_t)-1;  /* flag as a stop word */
     }    
 
     if (sw->verbose)
@@ -1997,7 +2022,7 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
             while (epi)
             {
                 /* If it is not a stopword write it */
-                if (epi->u1.wordID > 0)  
+                if (epi->u1.wordID > (sw_off_t)0)  
                     DB_WriteWordHash(sw, epi->word,epi->u1.wordID,indexf->DB);
                 epi = epi->next;
             }
@@ -2046,7 +2071,7 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
                         lastPercent = percent;
                     }
                 }
-                if (epi->u1.wordID > 0)   /* Not a stopword */
+                if (epi->u1.wordID > (sw_off_t)0)   /* Not a stopword */
                 {
                     build_worddata(sw, epi);
                     write_worddata(sw, epi, indexf);
@@ -2613,8 +2638,8 @@ void    coalesce_word_locations(SWISH * sw, ENTRY *e)
     unsigned char *buffer;
     unsigned int sz_buffer;
     unsigned char *coalesced_buffer;
-    int     *posdata;
-    int      local_posdata[MAX_STACK_POSITIONS];
+    unsigned int     *posdata;
+    unsigned int      local_posdata[MAX_STACK_POSITIONS];
 
 
     /* Check for new locations in the current chunk */
@@ -2731,7 +2756,7 @@ void    coalesce_word_locations(SWISH * sw, ENTRY *e)
         }
 
         if(frequency > MAX_STACK_POSITIONS)
-            posdata = emalloc(frequency * sizeof(int));
+            posdata = (unsigned int *)emalloc(frequency * sizeof(int));
         else
             posdata = local_posdata;
 
@@ -2849,13 +2874,13 @@ static void unSwapLocData(SWISH * sw, int idx_swap_file, ENTRY *ep)
     /* Reopen in read mode for (for faster reads, I suppose) */
     if(!idx->fp_loc_read[idx_swap_file])
     {
-        if (!(idx->fp_loc_read[idx_swap_file] = fopen(idx->swap_location_name[idx_swap_file], F_READ_BINARY)))
+        if (!(idx->fp_loc_read[idx_swap_file] = sw_fopen(idx->swap_location_name[idx_swap_file], F_READ_BINARY)))
             progerrno("Could not open temp file %s: ", idx->swap_location_name[idx_swap_file]);
     }
     else
     {
         /* File already opened for read -> reset pointer */
-        fseek(idx->fp_loc_read[idx_swap_file],0,SEEK_SET);
+        sw_fseek(idx->fp_loc_read[idx_swap_file],(sw_off_t)0,SEEK_SET);
     }
 
     fp = idx->fp_loc_read[idx_swap_file];
@@ -2889,7 +2914,7 @@ static void unSwapLocData(SWISH * sw, int idx_swap_file, ENTRY *ep)
             else
             {
                 /* Just advance file pointer */
-                idx->swap_seek(fp,lenbuf - sizeof(ENTRY *),SEEK_CUR);
+                idx->swap_seek(fp,(sw_off_t)(lenbuf - sizeof(ENTRY *)),SEEK_CUR);
             }
         }
     }
