@@ -17,11 +17,20 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-/*
-** use _AP() for easier cross-compiler (non-ANSI) porting 
-** <return value> <functionname> _AP( (<arg prototypes>) );
-*/
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-struct swline *addswline _AP ((struct swline *, char *));
-struct filter *addfilter _AP((struct filter *rp, char *FilterSuffix, char *FilterProg, char *FilterDir)); 
+struct swline *newswline_n(char *line, int size);
+struct swline *newswline(char *line);
+struct swline *addswline (struct swline *rp, char *line);
+struct swline *dupswline (struct swline *rp);
+void addindexfile(struct SWISH *sw, char *line);
+void freeswline (struct swline *ptr);
+void init_header (INDEXDATAHEADER *header);
+void free_header (INDEXDATAHEADER *header);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
