@@ -39,6 +39,7 @@
 #include "result_output.h"
 #include "parse_conffile.h"
 #include "stemmer.h"
+#include "headers.h"
 
 static void dup_header( SWISH *sw_input, SWISH *sw_output );
 static void check_header_match( IndexFILE *in_index, SWISH *sw_output );
@@ -48,7 +49,7 @@ static IndexFILE *get_next_file_in_order( SWISH *sw_input );
 static void add_file( FILE *filenum_map, IndexFILE *cur_index, SWISH *sw_output );
 static int *get_map( FILE *filenum_map, IndexFILE *cur_index );
 static void dump_index_words(SWISH * sw, IndexFILE * indexf, SWISH *sw_output );
-static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_map, int filenum, ENTRY *e, int metaID, int posdata );
+static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_map, int filenum, ENTRY *e, int metaID, unsigned int posdata );
 
 
 // #define DEBUG_MERGE
@@ -75,13 +76,13 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
                  frequency,
                  loc_count = 0,
                  word_count = 0;
-    long         wordID;
+    sw_off_t     wordID;
     int          metadata_length = 0;
     unsigned char   *worddata;
     unsigned char   *s, *start;
     unsigned char   flag;
-    int          local_posdata[MAX_STACK_POSITIONS];
-    int         *posdata;
+    unsigned int          local_posdata[MAX_STACK_POSITIONS];
+    unsigned int         *posdata;
     int          i;
 
     /*******************************************************************************
@@ -101,14 +102,14 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
             /* Duplicate the first index's header into the output index */
             dup_header( sw_input, sw_output );
         else
-            check_header_match( cur_index, sw_output );  // errors if headers don't match - don't really need to check first one since it was the one that was dupped 
+            check_header_match( cur_index, sw_output );  // errors if headers don't match - don't really need to check first one since it was the one that was dupped
 
 
         make_meta_map( cur_index, sw_output);        // add metas to new index, and create map
 
         load_filename_sort( sw_input, cur_index );   // so can read in filename order
 
-        cur_index->current_file = 0;  
+        cur_index->current_file = 0;
         cur_index->cur_prop = NULL;
 
 #ifdef DEBUG_MERGE
@@ -121,8 +122,9 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
 
 
 #ifdef DEBUG_MERGE
-    printf("----- Output Header ----------\n");
-    resultPrintHeader(sw_output, 0, &sw_output->indexlist->header, sw_output->indexlist->line, 0);
+    printf("----- Output Header (requires -H9) ----------\n");
+    print_index_headers( sw_output->indexlist );
+    printf("\n\n");
 #endif
 
 
@@ -222,10 +224,10 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
                             filenum += tmpval;
                             /* Use stack array when possible to avoid malloc/free overhead */
                             if(frequency > MAX_STACK_POSITIONS)
-                                posdata = (int *) emalloc(frequency * sizeof(int));
+                                posdata = (unsigned int *) emalloc(frequency * sizeof(int));
                             else
                                 posdata = local_posdata;
-                            
+
                             /* Read the positions */
                             uncompress_location_positions(&s,flag,frequency,posdata);
 
@@ -238,14 +240,14 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
                             {
                                 /* 08/2002 jmruiz - We will call CompressCurrentLocEntry from time
                                 ** to time to help addentry.
-                                ** If we do not do this, addentry routine will have to run linked lists 
+                                ** If we do not do this, addentry routine will have to run linked lists
                                 ** of positions with thousands of elements and makes the merge proccess
                                 ** very slow
                                 */
                                 if(!(loc_count % 100))
                                     CompressCurrentLocEntry(sw_output, e);
                             }
-                        
+
 
                             if(posdata != local_posdata)
                                 efree(posdata);
@@ -259,7 +261,7 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
                             {
                                 filenum = 0;
                                 metaID = uncompress2(&s);
-                                metadata_length = uncompress2(&s); 
+                                metadata_length = uncompress2(&s);
                                 start = s;
                             }
                         }
@@ -271,7 +273,7 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
                     }
                     cur_index = cur_index->next;
                 }
-                /* Let's coalesce locations for each word to save memory 
+                /* Let's coalesce locations for each word to save memory
                 ** This makes use of the -e feature
                 ** Because we are proccessing one word at a time we can
                 ** coalesce its data just once
@@ -304,8 +306,8 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
 
 
 #ifdef DEBUG_MERGE
-    printf("----- Final Output Header ----------\n");
-    resultPrintHeader(sw_output, 0, &sw_output->indexlist->header, sw_output->indexlist->line, 0);
+    printf("----- Final Output Header (requires -H9) ----------\n");
+    print_index_headers( sw_output->indexlist );
 #endif
 
     remove( tmpfilename );
@@ -355,7 +357,7 @@ void merge_indexes( SWISH *sw_input, SWISH *sw_output )
 *  rereads the header from the data base, and clears out some values
 *
 *****************************************************************************/
-       
+
 static void dup_header( SWISH *sw_input, SWISH *sw_output )
 {
     INDEXDATAHEADER *out_header = &sw_output->indexlist->header;
@@ -371,6 +373,8 @@ static void dup_header( SWISH *sw_input, SWISH *sw_output )
 
     freeMetaEntries( out_header );
 
+    /* Remove the date from the index */
+
     if ( out_header->indexedon )
     {
         efree( out_header->indexedon );
@@ -378,7 +382,7 @@ static void dup_header( SWISH *sw_input, SWISH *sw_output )
         out_header->lenindexedon = 0;
     }
 }
-    
+
 /****************************************************************************
 *  check_header_match -- makes sure that the imporant settings match
 *
@@ -400,18 +404,18 @@ static void compare_header( char *index, char *name, void *in, void *out )
     if ( in_item->len != out_item->len )
         progerr("Header %s in index %s doesn't match length in length with output header", name, index );
 
-    if ( strcmp( (const char *)in_item->str, (const char *)out_item->str )) 
+    if ( strcmp( (const char *)in_item->str, (const char *)out_item->str ))
         progerr("Header %s in index %s doesn't match output header", name, index );
-    
+
     //if ( memcmp( (const void *)in_item->str, (const void *)out_item->str, in_item->len ) )
     //    progerr("Header %s in index %s doesn't match output header", name, index );
-        
 
-        
+
+
 
 }
 
-   
+
 static void check_header_match( IndexFILE *in_index, SWISH *sw_output )
 {
     INDEXDATAHEADER *out_header = &sw_output->indexlist->header;
@@ -460,7 +464,6 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
     struct metaEntry *in_meta;
     struct metaEntry *out_meta;
     int             *meta_map;
-    
 
     meta_map = emalloc( sizeof( int ) * (in_header->metaCounter + 1) );
     memset( meta_map, 0, sizeof( int ) * (in_header->metaCounter + 1) );
@@ -475,35 +478,54 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
                    ? getMetaNameByNameNoAlias( out_header, in_meta->metaName )
                    : getPropNameByNameNoAlias( out_header, in_meta->metaName );
 
-        /* if it's not found, then add it */
+
+
+        /* if meta from input header is not found in the output header then add it */
         if ( !out_meta )
-            out_meta = addMetaEntry(out_header, in_meta->metaName, in_meta->metaType, 0);
-        else
-            if (out_meta->metaType != in_meta->metaType )
-                progerr("meta name %s in index %s is different type than in output index", in_meta->metaName, in_index->line );
+            out_meta = cloneMetaEntry( out_header, in_meta ); /* can't fail */
+
+
+        /* Validate that the two metas are indeed the same */
+        /* This should be done in metanames.c, but error messages are harder */
+
+        if (out_meta->metaType != in_meta->metaType )
+            progerr("meta name %s in index %s is different type than in output index", in_meta->metaName, in_index->line );
+
+        if (out_meta->sort_len != in_meta->sort_len )
+            progerr("meta name %s in index %s has different sort length than in output index", in_meta->metaName, in_index->line );
+
+        if (out_meta->rank_bias != in_meta->rank_bias )
+            progerr("meta name %s in index %s is different rank bias than in output index", in_meta->metaName, in_index->line );
+
+
 
 
         /* Now, save the mapping */
-        meta_map[ in_meta->metaID ] = out_meta->metaID;                
+        meta_map[ in_meta->metaID ] = out_meta->metaID;
 
 
-        /* now here's a pain, and lots of room for screw up. */
-        /* Basically, check for alias mappings, and that they are correct */
-        /* you can say title is an alias for swishtitle in one index, and then say */
-        /* title is an alias for doctitle in another index */
+        /* 
+         * now here's a pain, and lots of room for screw up.
+         * Basically, check for alias mappings, and that they are correct
+         * you can say title is an alias for swishtitle in one index, and then say
+         * title is an alias for doctitle in another index, which would be an error.
+         * So, if title is an alias for swishtitle, then the output index either
+         * needs to have that alias already, or it must be created.
+         */
 
-        /* If it's an alias, then make that mapping, too */
         if ( in_meta->alias )
         {
             struct metaEntry *in_alias;
             struct metaEntry *out_alias;
 
             /* Grab alias meta entry so we can look it up in the out_header */
-            
+
             in_alias = is_meta_index( in_meta )
                    ? getMetaNameByID( in_header, in_meta->alias )
                    : getPropNameByID( in_header, in_meta->alias );
 
+
+            /* This should not happen -- it would be a very broken input header */
             if ( !in_alias )
                 progerr("Failed to lookup alias for %s in index %s", in_meta->metaName, in_index->line );
 
@@ -514,7 +536,11 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
                    : getPropNameByNameNoAlias( out_header, in_alias->metaName );
 
 
-            /* should be there, since it would have been added earlier - the real metas must be added before the aliases */
+            /* 
+             * should be there, since it would have been added earlier 
+             * the real metas must be added before the aliases 
+             * */
+
             if ( !out_alias )
                 progerr("Failed to lookup alias for %s in output index", out_meta->metaName );
 
@@ -525,7 +551,14 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
 
             /* else, if it is already an alias, but points someplace else, we have a problem */
             else if ( out_meta->alias != out_alias->metaID )
-                progerr("In index %s metaname '%s' is an alias for '%s'(%d).  But another index already mapped '%s' to ID# '%d'", in_index->line, in_meta->metaName, in_alias->metaName, in_alias->metaID, out_meta->metaName, out_meta->alias );
+                progerr("In index %s metaname '%s' is an alias for '%s'(%d).  But another input index already mapped '%s' to '%s'(%d)", 
+                        in_index->line, in_meta->metaName, in_alias->metaName, in_alias->metaID,
+                        out_meta->metaName,
+                        is_meta_index( out_meta )
+                            ?  getMetaNameByID( out_header,  out_meta->alias )->metaName
+                            :  getPropNameByID( out_header,  out_meta->alias )->metaName,
+                        out_meta->alias
+                        );
         }
     }
 
@@ -537,7 +570,7 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
     for ( i=0; i<in_header->metaCounter + 1;i++)
         printf("%4d  ->  %3d\n", i, meta_map[i] );
 #endif
-        
+
 }
 
 /****************************************************************************
@@ -556,7 +589,7 @@ static int     compnums(const void *s1, const void *s2)
     int         v2 = sorted_data[ b-1 ];
 
     // return v1 <=> v2;
-    
+
     if ( v1 < v2 )
         return -1;
     if ( v1 > v2 )
@@ -587,7 +620,7 @@ static void load_filename_sort( SWISH *sw, IndexFILE *cur_index )
 
     cur_index->modified_meta = getPropNameByName( &cur_index->header, AUTOPROPERTY_LASTMODIFIED );
 
-        
+
     if ( !LoadSortedProps( cur_index, path_meta ) )
     {
         FileRec fi;
@@ -632,17 +665,19 @@ static void load_filename_sort( SWISH *sw, IndexFILE *cur_index )
 
 static void print_file_removed(IndexFILE *older, propEntry *op, IndexFILE *newer, propEntry *np )
 {
-    
+
     char *p1, *d1, *p2, *d2;
     p1 = DecodeDocProperty( older->path_meta, older->cur_prop );
     d1 = DecodeDocProperty( older->modified_meta, op );
-    
+
     p2 = DecodeDocProperty( newer->path_meta, newer->cur_prop );
     d2 = DecodeDocProperty( newer->modified_meta, np );
-    
-    printf("Replaced file '%s %s' with '%s %s'\n",
+
+    printf("Replaced file '%s:%s %s' with '%s:%s %s'\n",
+         older->line,
          *p1 ? p1 : "(file name not defined)",
          *d1 ? d1 : "(date not defined)",
+         newer->line,
          *p2 ? p2 : "(file name not defined)",
          *d2 ? d2 : "(date not defined)"
     );
@@ -693,7 +728,7 @@ static IndexFILE *get_next_file_in_order( SWISH *sw_input )
 
         ret = Compare_Properties( cur_index->path_meta, cur_index->cur_prop, winner->cur_prop );
 
-        if ( ret != 0 )  
+        if ( ret != 0 )
         {
             if ( ret < 0 )  /* take cur_index if it's smaller */
                 winner = cur_index;
@@ -756,11 +791,11 @@ static IndexFILE *get_next_file_in_order( SWISH *sw_input )
     if ( !winner )
         return NULL;
 
-        
+
     winner->filenum = winner->path_order[winner->current_file++];
 
 #ifdef DEBUG_MERGE
-printf("   Files in order: index %s file# %d winner\n", winner->line, winner->filenum );            
+printf("   Files in order: index %s file# %d winner\n", winner->line, winner->filenum );
 #endif
 
     /* free prop, as it's not needed anymore */
@@ -772,7 +807,7 @@ printf("   Files in order: index %s file# %d winner\n", winner->line, winner->fi
     return winner;
 }
 
-        
+
 /****************************************************************************
 *  add_file
 *
@@ -814,7 +849,7 @@ static void add_file( FILE *filenum_map, IndexFILE *cur_index, SWISH *sw_output
     fi.docProperties = d;
     dump_file_properties( cur_index, &fi );
 #endif
-    
+
 
 
     /* all this off-by-one things are a mess */
@@ -834,7 +869,7 @@ static void add_file( FILE *filenum_map, IndexFILE *cur_index, SWISH *sw_output
     printf("\n");
 #endif
 
-   
+
     /* Now bump the file counter  */
     idx->filenum++;
     indexf->header.totalfiles++;
@@ -849,7 +884,7 @@ static void add_file( FILE *filenum_map, IndexFILE *cur_index, SWISH *sw_output
         freeDocProperties( d );
     }
 
-    
+
 
 
     /* now write out the data to be used for mapping file for a given index. */
@@ -892,7 +927,7 @@ static int *get_map( FILE *filenum_map, IndexFILE *cur_index )
     int         filenum;
     int         new_filenum = 0;
 
-    
+
 
     memset( array, 0, (cur_index->header.totalfiles+1) * sizeof( int ) );
 
@@ -906,8 +941,8 @@ static int *get_map( FILE *filenum_map, IndexFILE *cur_index )
 
         if (!fread( &filenum, sizeof(int), 1, filenum_map))
             break;
-        
-        
+
+
         if(!fread( &idf, sizeof(IndexFILE *), 1, filenum_map))
             break;
 
@@ -922,24 +957,24 @@ static int *get_map( FILE *filenum_map, IndexFILE *cur_index )
 /****************************************************************************
 *  Reads the index to get the all the words
 ****************************************************************************/
-   
+
 static void dump_index_words(SWISH * sw, IndexFILE * indexf, SWISH *sw_output)
 {
     int         j;
     int         word_count = 0;
     char        word[2];
     char       *resultword;
-    long        wordID;
+    sw_off_t    wordID;
 
     DB_InitReadWords(sw, indexf->DB);
 
 
     printf("Getting words in index '%s': %3d words\r", indexf->line, word_count);
     fflush(stdout);
-    
+
     for(j=0;j<256;j++)
     {
-        
+
         word[0] = (unsigned char) j; word[1] = '\0';
         DB_ReadFirstWordInvertedIndex(sw, word,&resultword,&wordID,indexf->DB);
 
@@ -966,14 +1001,14 @@ static void dump_index_words(SWISH * sw, IndexFILE * indexf, SWISH *sw_output)
 *
 ****************************************************************************/
 
-static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_map, int filenum, ENTRY *e, int metaID, int posdata )
+static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_map, int filenum, ENTRY *e, int metaID, unsigned int posdata )
 {
     int         new_file;
     int         new_meta;
 
 #ifdef DEBUG_MERGE
     printf("\nindex %s '%s' Struct: %d Pos: %d",
-    indexf->line, e->word, structure, position );
+    indexf->line, e->word, GET_STRUCTURE(posdata), GET_POSITION(posdata) );
 
 
     if ( !(new_file = file_num_map[ filenum ]) )
@@ -990,7 +1025,7 @@ static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_m
 
     printf("  File: %d -> %d  Meta: %d -> %d\n", filenum, new_file, metaID, new_meta );
 
-    addentry( sw_output, e, new_file, structure, metaID, position );
+    addentry( sw_output, e, new_file, GET_STRUCTURE(posdata), new_meta, GET_POSITION(posdata) );
 
     return;
 
@@ -1004,7 +1039,7 @@ static void write_word_pos( IndexFILE *indexf, SWISH *sw_output, int *file_num_m
     if ( !(new_meta = indexf->meta_map[ metaID ] ))
         return;
 
-    addentry( sw_output, e, new_file, GET_STRUCTURE(posdata), metaID, GET_POSITION(posdata) );
+    addentry( sw_output, e, new_file, GET_STRUCTURE(posdata), new_meta, GET_POSITION(posdata) );
 
     return;
 
