@@ -12,6 +12,11 @@ extern struct _indexing_data_source_def FileSystemIndexingDataSource;
 extern struct _indexing_data_source_def HTTPIndexingDataSource;
 #endif
 
+#ifdef ALLOW_EXTERNAL_PROGRAM_DATA_SOURCE
+extern struct _indexing_data_source_def ExternalProgramDataSource;
+#endif    
+
+
 struct _indexing_data_source_def *data_sources[] = {
 
 #ifdef ALLOW_FILESYSTEM_INDEXING_DATA_SOURCE
@@ -22,5 +27,9 @@ struct _indexing_data_source_def *data_sources[] = {
     &HTTPIndexingDataSource,
 #endif
 
+#ifdef ALLOW_EXTERNAL_PROGRAM_DATA_SOURCE
+    &ExternalProgramDataSource,
+#endif    
+
     0
 };
