@@ -6,11 +6,11 @@ int ramdisk_close(FILE *);
 
 void add_buffer_ramdisk(struct ramdisk *);
 
-long ramdisk_tell(FILE *);
+sw_off_t ramdisk_tell(FILE *);
 
 size_t ramdisk_write(const void *,size_t, size_t, FILE *);
 
-int ramdisk_seek(FILE *,long, int );
+int ramdisk_seek(FILE *,sw_off_t, int );
 
 size_t ramdisk_read(void *, size_t, size_t, FILE *);
 
