@@ -1,5 +1,5 @@
 /*
-$Id: search.c,v 1.129.2.1 2003/12/23 15:26:44 whmoseley Exp $
+$Id: search.c,v 1.137 2004/09/20 22:01:57 karman Exp $
 **
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
@@ -154,6 +154,22 @@ static void freeresult(RESULT *);
 static void make_db_res_and_free(RESULT_LIST *l_res);
 
 
+/**********************************************************************
+* SwishRankScheme -- set the ranking scheme to use when sorting
+* karman - Wed Sep  1 11:55:46 CDT 2004
+*
+***********************************************************************/
+
+void SwishRankScheme(SWISH *sw, int scheme)
+{
+	
+    sw->RankScheme = scheme;
+    
+}
+
+
+
+
 /****************************************************************
 *  New_Search_Object - Create a new search object
 *
@@ -279,7 +295,10 @@ void SwishSetSort( SEARCH_OBJECT *srch, char *sort )
         return;
 
     if ( srch->sort_params )
+    {
         freeswline( srch->sort_params );
+        srch->sort_params = NULL;
+    }
 
 
     if ( !(slsort = parse_line(sort)) )
@@ -851,6 +870,7 @@ RESULTS_OBJECT *SwishExecute(SEARCH_OBJECT *srch, char *words)
 }
 
 
+
 /**************************************************************************
 *  limit_result_list -- removes results that are not within the limit
 *
@@ -1159,8 +1179,8 @@ static RESULT_LIST *parseterm(DB_RESULTS *db_results, int parseone, int metaID,
      * The andLevel is used to help keep the ranking function honest
      * when it ANDs the results of the latest search term with
      * the results so far (rp).  The idea is that if you AND three
-     * words together you ultimately want to resulting rank to
-     * be the average of all three individual work ranks. By keeping
+     * words together you ultimately want the resulting rank to
+     * be the average of all three individual ranks. By keeping
      * a running total of the number of terms already ANDed, the
      * next AND operation can properly scale the average-rank-so-far
      * and recompute the new average properly (see andresultlists()).
@@ -1401,7 +1421,7 @@ static void addResultToList(RESULT_LIST *l_r, RESULT *r)
 
 /* Routine to test structure in a result */
 /* Also removes posdata that do not fit with structure field */
-static int test_structure(int structure, int frequency, int *posdata)
+static int test_structure(int structure, int frequency, unsigned int *posdata)
 {
     int i, j;    /* i -> counter upto frequency, j -> new frequency */
     int *p,*q;   /* Use pointers to ints instead of arrays for
@@ -1447,15 +1467,15 @@ static RESULT_LIST *getfileinfo(DB_RESULTS *db_results, char *word, int metaID)
             index_structfreq,
             tmpval;
     RESULT_LIST *l_rp, *l_rp2;
-    long    wordID;
+    sw_off_t    wordID;
     int     metadata_length;
     char   *p;
     int     tfrequency = 0;
     unsigned char   *s, *buffer; 
     int     sz_buffer;
     unsigned char flag;
-    int     stack_posdata[MAX_POSDATA_STACK];  /* stack buffer for posdata */
-    int    *posdata;
+    unsigned int     stack_posdata[MAX_POSDATA_STACK];  /* stack buffer for posdata */
+    unsigned int    *posdata;
     IndexFILE  *indexf = db_results->indexf;
     SWISH  *sw = indexf->sw;
     int     structure = db_results->srch->structure;
@@ -1585,7 +1605,7 @@ static RESULT_LIST *getfileinfo(DB_RESULTS *db_results, char *word, int metaID)
                 /* stack_posdata is just to avoid calling emalloc */
                 /* it should be enough for most cases */
                 if(frequency > MAX_POSDATA_STACK)
-                    posdata = (int *)emalloc(frequency * sizeof(int));
+                    posdata = (unsigned int *)emalloc(frequency * sizeof(int));
                 else
                     posdata = stack_posdata;
 
@@ -1605,9 +1625,11 @@ static RESULT_LIST *getfileinfo(DB_RESULTS *db_results, char *word, int metaID)
                     if(!l_rp)
                        l_rp = newResultsList(db_results);
 
-                    // tfrequency = number of files with this word
-                    // frequency = number of times this words is in this document for this metaID
-                    // metarank is the negative of the metaID - for use in getrank()
+                    /*
+		       tfrequency = number of files with this word
+                       frequency = number of times this words is in this document for this metaID
+                       metarank is the negative of the metaID - for use in getrank()
+		    */
 
                     addtoresultlist(l_rp, filenum, meta_rank, tfrequency, frequency, db_results);
 
@@ -1756,8 +1778,12 @@ static RESULT_LIST *andresultlists(DB_RESULTS *db_results, RESULT_LIST * l_r1, R
              */
             int     newRank = 0;
 
-
             newRank = ((r1->rank * andLevel) + r2->rank) / (andLevel + 1);
+
+
+#ifdef DEBUG_RANK
+    fprintf( stderr, "\n----\nFile num: %d  1st score: %d  2nd score: %d  andLevel: %d  newRank:  %d\n----\n", r1->filenum, r1->rank, r2->rank, andLevel, newRank );
+#endif
             
 
             if(!new_results_list)
@@ -1851,7 +1877,12 @@ static RESULT_LIST *orresultlists(DB_RESULTS *db_results, RESULT_LIST * l_r1, RE
 
             rp->fi.filenum = rp->filenum = r1->filenum;
 
-            rp->rank = ( r1->rank + r2->rank) * 2;  // bump up the or terms
+            rp->rank = ( r1->rank + r2->rank) * 2;  /* bump up the or terms */
+	    
+#ifdef DEBUG_RANK
+    fprintf( stderr, "\n----\nFile num: %d  1st score: %d  2nd score: %d  newRank:  %d\n----\n", r1->filenum, r1->rank, r2->rank, rp->rank );
+#endif
+
             rp->tfrequency = 0;
             rp->frequency = r1->frequency + r2->frequency;
             rp->db_results = r1->db_results;
@@ -2313,7 +2344,7 @@ static RESULT_LIST *notresultlists(DB_RESULTS *db_results, RESULT_LIST * l_r1, R
 /* This routine is used by qsort */
 static int     icomp_posdata(const void *s1, const void *s2)
 {
-    return (GET_POSITION(*(int *) s1) - GET_POSITION(*(int *) s2));
+    return (GET_POSITION(*(unsigned int *) s1) - GET_POSITION(*(unsigned int *) s2));
 }
 
 
