@@ -40,14 +40,26 @@ const char **SwishIndexNames( SW_HANDLE );  /* fetch list of index files names a
 SWISH_HEADER_VALUE SwishHeaderValue( SW_HANDLE, const char *index_name, const  char *cur_header, SWISH_HEADER_TYPE *type );
 SWISH_HEADER_VALUE SwishResultIndexValue( SW_RESULT, const char *name, SWISH_HEADER_TYPE *type );
 
+typedef const void * SW_META;
+typedef SW_META * SWISH_META_LIST;
 
+/* Meta and Property Values */
 
+#define SW_META_TYPE_UNDEF 0
+#define SW_META_TYPE_STRING 4
+#define SW_META_TYPE_ULONG 8
+#define SW_META_TYPE_DATE 16
 
-
+SWISH_META_LIST SwishMetaList( SW_HANDLE, const char *index_name );
+SWISH_META_LIST SwishPropertyList( SW_HANDLE, const char *index_name );
+SWISH_META_LIST SwishResultMetaList( SW_RESULT );
+SWISH_META_LIST SwishResultPropertyList( SW_RESULT );
+const char *SwishMetaName( SW_META );
+int SwishMetaType( SW_META );
+int SwishMetaID( SW_META );
 
 /* Limit searches by structure */
 
-
 #define IN_FILE_BIT     0
 #define IN_TITLE_BIT    1
 #define IN_HEAD_BIT     2
@@ -76,7 +88,10 @@ SW_RESULTS SwishQuery(SW_HANDLE, char *words );
 
 SW_SEARCH New_Search_Object( SW_HANDLE, char *query );
 
+void SwishRankScheme( SW_HANDLE sw, int scheme );
 void SwishSetRefPtr( SW_HANDLE sw, void *address );
+void *SwishGetRefPtr( SW_HANDLE sw );
+
 void *SwishSearch_parent( SW_SEARCH srch );
 void *SwishResults_parent( SW_RESULTS results );
 void *SwishResult_parent( SW_RESULT result );
@@ -133,6 +148,7 @@ const char *SwishWordsByLetter(SW_HANDLE, char *filename, char c);
 char *SwishStemWord( SW_HANDLE, char *word );  /* Really this is depreciated */
 
 SW_FUZZYWORD SwishFuzzyWord( SW_RESULT r, char *word );
+SW_FUZZYWORD SwishFuzzy( SW_HANDLE sw, const char *index_name, char *word );
 const char **SwishFuzzyWordList( SW_FUZZYWORD fw );
 int SwishFuzzyWordCount( SW_FUZZYWORD fw );
 int SwishFuzzyWordError( SW_FUZZYWORD fw );
