@@ -1,5 +1,5 @@
 /*
-$Id: swish.c,v 1.116.2.3 2004/01/30 19:47:18 whmoseley Exp $
+$Id: swish.c,v 1.126 2004/12/06 22:08:43 whmoseley Exp $
 **
 ** Swish Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
@@ -80,6 +80,7 @@ static DEBUG_MAP debug_map[] = {
     {"INDEX_WORDS_FULL", DEBUG_INDEX_WORDS_FULL, "List words stored in index (more verbose)"},
     {"INDEX_STOPWORDS", DEBUG_INDEX_STOPWORDS, "List stopwords stored in index"},
     {"INDEX_FILES", DEBUG_INDEX_FILES, "List file data stored in index"},
+    {"INDEX_WORD_COUNT", DEBUG_INDEX_WORD_COUNT, "List number of words in all files"},
     {"INDEX_METANAMES", DEBUG_INDEX_METANAMES, "List metaname table stored in index"},
     {"INDEX_ALL", DEBUG_INDEX_ALL, "Dump data ALL above data from index file"},
     {"LIST_FUZZY_MODES", DEBUG_LIST_FUZZY, "List fuzzy options available\n\n-- indexing --\n"},
@@ -94,22 +95,11 @@ static DEBUG_MAP debug_map[] = {
 };
 
 
-/* Possible run modes */
-typedef enum {
-    MODE_SEARCH,
-    MODE_INDEX,
-    MODE_DUMP,
-    MODE_WORDS,
-    MODE_MERGE,
-    MODE_UPDATE
-}
-CMD_MODE;
-
 
 /* Parameters read from the command line, that are not stored in *SWISH */
 typedef struct
 {
-    CMD_MODE    run_mode;           /* selected run mode */
+    CMD_MODE    run_mode;           /* selected run mode.  Default is MODE_SEARCH */
     char        keychar;            /* for dumping words */
 
 
@@ -119,7 +109,7 @@ typedef struct
     int          structure;         /* Structure for limiting to HTML tags */
     struct swline *sort_params;     /* sort properties */
     LIMIT_PARAMS *limit_params;     /* for storing -L command line settings */
-    
+
     int         query_len;          /* length of buffer */
 
     struct swline *disp_props;      /* extra display props */
@@ -188,7 +178,7 @@ int     main(int argc, char **argv)
     /* I don't like this.   modules.c would fix this */
     IndexingDataSource = data_sources[0];
 
-    
+
 
     params = new_swish_params();
     get_command_line_params(sw, argv, params );
@@ -206,6 +196,7 @@ int     main(int argc, char **argv)
 
         case MODE_INDEX:
         case MODE_UPDATE:
+        case MODE_REMOVE:
             cmd_index( sw, params );
             break;
 
@@ -217,7 +208,7 @@ int     main(int argc, char **argv)
             cmd_keywords( sw ,params );  /* -k setting */
             break;
 
-            
+
         default:
             progerr("Invalid operation mode '%d'", (int)params->run_mode);
     }
@@ -265,7 +256,7 @@ static void    usage()
     printf("    swish [-e] [-i dir file ... ] [-S system] [-c file] [-f file] [-l] [-v (num)]\n");
     printf("    swish -w word1 word2 ... [-f file1 file2 ...] \\\n");
     printf("          [-P phrase_delimiter] [-p prop1 ...] [-s sortprop1 [asc|desc] ...] \\\n");
-    printf("          [-m num] [-t str] [-d delim] [-H (num)] [-x output_format]\n");
+    printf("          [-m num] [-t str] [-d delim] [-H (num)] [-x output_format] [-R rank_scheme]\n");
     printf("    swish -k (char|*) [-f file1 file2 ...]\n");
     printf("    swish -M index1 index2 ... outputfile\n");
     printf("    swish -N /path/to/compare/file\n");
@@ -273,6 +264,38 @@ static void    usage()
     putchar('\n');
     printf("options: defaults are in brackets\n");
 
+    printf("         -b : begin results at this number\n");
+    printf("         -c : configuration file(s) to use for indexing\n");
+    printf("         -d : next param is delimiter.\n");
+    printf("         -E : Append errors to file specified, or stderr if file not specified.\n");
+    printf("         -e : \"Economic Mode\": The index proccess uses less RAM.\n");
+    printf("         -f : index file to create or file(s) to search from [%s]\n", INDEXFILE);
+    printf("         -H : \"Result Header Output\": verbosity (0 to 9)  [1].\n");
+    printf("         -i : create an index from the specified files\n");
+
+#ifdef ALLOW_FILESYSTEM_INDEXING_DATA_SOURCE
+    printf("              for \"-S fs\" - specify a list of files or directories\n");
+#endif
+
+#ifdef ALLOW_HTTP_INDEXING_DATA_SOURCE
+    printf("              for \"-S http\" - specify a list of URLs\n");
+#endif
+
+#ifdef ALLOW_EXTERNAL_PROGRAM_DATA_SOURCE
+    printf("              for \"-S prog\" - specify a list of programs or the string \"stdin\"\n");
+#endif
+
+    printf("         -k : Print words starting with a given char.\n");
+    printf("         -l : follow symbolic links when indexing\n");
+    printf("         -M : merges index files\n");
+    printf("         -m : the maximum number of results to return [defaults to all results]\n");
+    printf("         -N : index only files with a modification date newer than path supplied\n");
+    printf("         -P : next param is Phrase delimiter.\n");
+    printf("         -p : include these document properties in the output \"prop1 prop2 ...\"\n");
+    printf("         -R : next param is Rank Scheme number (0 to 1)  [0].\n");
+#ifdef USE_BTREE
+        printf("         -r : remove: remove files from index\n");
+#endif
 
     printf("         -S : specify which indexing system to use.\n");
     printf("              Valid options are:\n");
@@ -294,35 +317,20 @@ static void    usage()
     if (!*defaultIndexingSystem)
         defaultIndexingSystem = "http";
 #endif
-
     printf("              The default value is: \"%s\"\n", defaultIndexingSystem);
 
-    printf("         -i : create an index from the specified files\n");
+
+    printf("         -s : sort by these document properties in the output \"prop1 prop2 ...\"\n");
+    printf("         -T : Trace options ('-T help' for info)\n");
+    printf("         -t : tags to search in - specify as a string\n");
+    printf("              \"HBthec\" - in Head|Body|title|header|emphasized|comments\n");
 #ifdef USE_BTREE
     printf("         -u : update: adds files to existing index\n");
 #endif
-    printf("         -w : search for words \"word1 word2 ...\"\n");
-    printf("         -t : tags to search in - specify as a string\n");
-    printf("              \"HBthec\" - in Head|Body|title|header|emphasized|comments\n");
-    printf("         -f : index file to create or file(s) to search from [%s]\n", INDEXFILE);
-    printf("         -c : configuration file(s) to use for indexing\n");
-    printf("         -v : indexing verbosity level (0 to 3) [-v %d]\n", VERBOSE);
-    printf("         -T : Trace options ('-T help' for info\n");
-    printf("         -l : follow symbolic links when indexing\n");
-    printf("         -b : begin results at this number\n");
-    printf("         -m : the maximum number of results to return [defaults to all results]\n");
-    printf("         -M : merges index files\n");
-    printf("         -N : index only files with a modification date newer than path supplied\n");
-    printf("         -p : include these document properties in the output \"prop1 prop2 ...\"\n");
-    printf("         -s : sort by these document properties in the output \"prop1 prop2 ...\"\n");
-    printf("         -d : next param is delimiter.\n");
-    printf("         -P : next param is Phrase delimiter.\n");
     printf("         -V : prints the current version\n");
-    printf("         -e : \"Economic Mode\": The index proccess uses less RAM.\n");
+    printf("         -v : indexing verbosity level (0 to 3) [-v %d]\n", VERBOSE);
+    printf("         -w : search for words \"word1 word2 ...\"\n");
     printf("         -x : \"Extended Output Format\": Specify the output format.\n");
-    printf("         -H : \"Result Header Output\": verbosity (0 to 9)  [1].\n");
-    printf("         -k : Print words starting with a given char.\n");
-    printf("         -E : Append errors to file specified, or stderr if file not specified.\n");
     printf("\n");
     printf("version: %s\n docs: http://swish-e.org\n Scripts and Modules at: (libexecdir) = %s\n", VERSION, get_libexec());
     exit(1);
@@ -337,8 +345,8 @@ static void    printversion()
 
 
 
-/* 
-  -- init swish structure 
+/*
+  -- init swish structure
 */
 
 /*************************************************************************
@@ -530,7 +538,7 @@ static void free_command_line_params( CMDPARAMS *params )
 {
     if ( params->disp_props )
         freeswline( params->disp_props );
-        
+
     if ( params->conflist )
         freeswline( params->conflist );
 
@@ -559,11 +567,11 @@ static char *is_another_param( char **argv )
             ? *(argv + 1)
             : NULL;
 }
-    
+
 static char *next_param( char ***argv )
 {
     char *c;
-    
+
     if ( ( c = is_another_param( *argv ) ) )
     {
         (*argv)++;
@@ -584,7 +592,7 @@ static int get_param_number(char ***argv, char c )
         progerr(" '-%c' requires a positive integer.", c );
 
     num = strtol( string, &badchar, 10 ); // would base zero be more flexible?
-    
+
     if ( num == LONG_MAX || num == LONG_MIN )
         progerrno("Failed to convert '-%c %s' to a number: ", c, string );
 
@@ -621,7 +629,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
     char c;
     char *w;
 
-    
+
 
 
     params->index_read_only = check_readonly_mode( *argv );
@@ -651,7 +659,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
         {
 
             /* Search related options */
-            
+
             case 'w':  /* query string */
             case 'L':  /* Limit range */
             case 'P':  /* phrase char */
@@ -664,6 +672,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
             case 'p':  /* old-style display properties */
             case 'd':  /* old-style custom delimiter */
             case 'o':  /* don't use pre-sorted indexes */
+            case 'R':  /* Ranking Scheme -- default is 1 */
                 argv = fetch_search_params( sw, argv, params, c );
                 break;
 
@@ -679,15 +688,15 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
                 argv = fetch_indexing_params( sw, argv, params, c );
                 break;
 
-            
+
 
             /* Index file(s) selection */
-        
+
             case 'f':
             {
                 if ( !is_another_param( argv ) )
                     progerr(" '-f' requires list of index files.");
-                
+
                 while ( (w = next_param( &argv )) )
                     addindexfile(sw, w);
 
@@ -702,7 +711,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
                 if ( !(w = next_param( &argv )) )
                     progerr(" '-k' requires a character (or '*').");
 
-                if ( strlen( w ) != 1 )                
+                if ( strlen( w ) != 1 )
                     progerr(" '-k' requires a character (or '*').");
 
 
@@ -714,7 +723,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
 
 
             /* print the version number */
-        
+
             case 'V':
                 printversion();
 
@@ -732,7 +741,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
             {
                 if ( !is_another_param( argv )  )
                     progerr(" '-M' requires an output file name.");
-                
+
                 params->run_mode = MODE_MERGE;
 
                 while ( (w = next_param( &argv )) )
@@ -775,7 +784,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
 
 
             /* Set where errors go */
-        
+
             case 'E':
             {
                 if ( !is_another_param( argv ) )
@@ -795,18 +804,35 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
                 break;
             }
 
-
-
-#ifdef USE_BTREE
-            /* Update mode jmruiz 2002/03 */
-
             case 'u':
+            case 'r':
+
+#ifndef USE_BTREE
+                progerr("Must compile swish-e with --enable-incremental to use -%c option",c);
+#else
             {
-                params->run_mode = MODE_UPDATE;
+                int mode = ( 'u' == c )
+                        ? MODE_UPDATE
+                        : MODE_REMOVE;
+
+
+                /* Make sure not trying to mix modes at same time */
+
+                if ( MODE_UPDATE == params->run_mode || MODE_REMOVE == params->run_mode )
+                    if ( mode != params->run_mode )
+                        progerr("Cannot mix -u (update) and -r (remove) indexing modes.");
+
+
+                params->run_mode = mode;
+
+                if ( is_another_param( argv ) )
+                    progerr("Option -%c does not take a parameter -- use -i to list paths", c );
+
                 break;
             }
 #endif
 
+
             default:
                 progerr("Unknown switch '-%c'.  Use -h for options.", c );
         }
@@ -822,7 +848,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
 static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, char switch_char )
 {
     char *w;
-    
+
     switch (switch_char)
     {
 
@@ -830,9 +856,11 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
         case 'i':
         {
             if ( !is_another_param( argv ) )
-                progerr(" '-i' requires a list of index files.");
+                progerr(" '-i' requires a list of things to index.");
 
-            if(params->run_mode != MODE_UPDATE)   /* Preserve update mode */
+            /* Set run_mode to index, unless in update/remove mode */
+
+            if ( MODE_UPDATE != params->run_mode && MODE_REMOVE != params->run_mode )
                 params->run_mode = MODE_INDEX;
 
             while ( (w = next_param( &argv )) )
@@ -854,7 +882,7 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
             for (data_source = data_sources; *data_source != 0; data_source++)
                 if (strcmp(w, (*data_source)->IndexingDataSourceId) == 0)
                     break;
-                    
+
 
             if (!*data_source)
                 progerr("Unknown -S option \"%s\"", w);
@@ -874,7 +902,10 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
             if ( !is_another_param( argv ) )
                 progerr(" '-c' requires one or more configuration files.");
 
-            if(params->run_mode != MODE_UPDATE)   /* Preserve update mode */     
+
+            /* Set one of the indexing modes when specifying -c */
+
+            if ( MODE_UPDATE != params->run_mode && MODE_REMOVE != params->run_mode )
                 params->run_mode = MODE_INDEX;
 
             while ( (w = next_param( &argv )) )
@@ -885,7 +916,7 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
 
 
         /* Follow symbolic links */
-        
+
         case 'l':
             sw->FS->followsymlinks = 1;
             break;
@@ -918,7 +949,7 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
 
 
         /* verbose while indexing */
-        
+
         case 'v':
         {
             params->hasverbose = 1;
@@ -926,7 +957,7 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
             break;
         }
 
-        
+
 
         default:
             progerr("Invalid index switch option '%s'", switch_char );
@@ -967,8 +998,8 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
                 params->query = (char *)emalloc( params->query_len + 1 );
                 params->query[0] = '\0';
             }
-                
-                
+
+
             while ( (w = next_param( &argv )) )
             {
                 /* don't add blank words */
@@ -1015,14 +1046,14 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
             params->PhraseDelimiter = (int) w[0];
             break;
         }
-        
+
 
         /* limit by structure */
 
         case 't':
         {
             char * c;
-            
+
             if ( !(w = next_param( &argv )) )
                 progerr("Specify tag fields (HBtheca).");
 
@@ -1058,7 +1089,7 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
                 }
             break;
         }
-        
+
 
         /* sort properties */
 
@@ -1066,7 +1097,7 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
         {
             if ( !is_another_param( argv ) )
                 progerr(" '-s' requires list of sort properties.");
-                
+
             while ( (w = next_param( &argv )) )
                 params->sort_params = addswline(params->sort_params, w);
 
@@ -1092,7 +1123,7 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
         /* $$$ These need better error reporting */
 
         /* Extended format */
-        
+
         case 'x':
         {
             /* Jose Ruiz 09/00 */
@@ -1127,11 +1158,11 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
         {
             if ( !is_another_param( argv ) )
                 progerr(" '-p' requires list of properties.");
-                
+
             while ( (w = next_param( &argv )) )
                 params->disp_props = addswline( params->disp_props, w);
 
-            break;                
+            break;
         }
 
 
@@ -1168,7 +1199,7 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
                             continue;
                         }
                     }
-                        
+
 
                     switch ( w[i] )
                     {
@@ -1199,6 +1230,10 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
         }
 
 
+        /* Ranking Scheme */
+        case 'R':
+            sw->RankScheme = get_param_number( &argv, switch_char );
+            break;
 
         /* Ignore sorted indexes */
 
@@ -1228,7 +1263,7 @@ static int check_readonly_mode( char *prog )
 
     if ( tmp < prog )
         return 0;
-    
+
     /* We must ignore case for WIN 32 */
     if (strcasecmp(tmp, "swish-search") == 0)
         return 1;
@@ -1238,7 +1273,7 @@ static int check_readonly_mode( char *prog )
 
 
 /*************************************************************************
-*   Dumps the index file(s) 
+*   Dumps the index file(s)
 *
 **************************************************************************/
 static void cmd_dump( SWISH *sw, CMDPARAMS *params )
@@ -1291,7 +1326,7 @@ static void cmd_index( SWISH *sw, CMDPARAMS *params )
 
 
     if (!hasdir)
-        progerr("Specify directories or files to index.");
+        progerr("Specify directories or files to %s.", MODE_INDEX == params->run_mode ? "index" : MODE_UPDATE == params->run_mode ? "update" : "remove" );
 
 
     if (sw->verbose < 0)
@@ -1302,19 +1337,23 @@ static void cmd_index( SWISH *sw, CMDPARAMS *params )
 
 
     /* Check for UPDATE_MODE jmruiz 2002/03 */
-    if(params->run_mode == MODE_UPDATE)
+    if ( MODE_UPDATE == params->run_mode || MODE_REMOVE == params->run_mode )
     {
+        /* Set update_mode */
+        sw->Index->update_mode = params->run_mode;
+
         /* Open the index file for read/write */
         sw->indexlist->DB = (void *) DB_Open(sw, sw->indexlist->line,DB_READWRITE);
         if ( sw->lasterror )
             SwishAbortLastError( sw );
-        
+
 
         /* Read the header and overwrite the '-c' option and default values -
         ** In other words, the header values are the good ones */
         read_header(sw, &sw->indexlist->header, sw->indexlist->DB);
         sw->TotalWords = sw->indexlist->header.totalwords;
         sw->TotalFiles = sw->indexlist->header.totalfiles;
+        sw->TotalWordPos = sw->indexlist->header.total_word_positions;
 
         /* Adjust filenum to totalfiles */
         sw->Index->filenum = sw->TotalFiles;
@@ -1322,17 +1361,15 @@ static void cmd_index( SWISH *sw, CMDPARAMS *params )
 #ifndef USE_BTREE
         progerr("Invalid operation mode '%d': Update mode only supported with USE_BTREE feature", (int)params->run_mode);
 #endif
-        /* Set update_mode */
-        sw->Index->update_mode = 1;
     }
+
+
     else
     {
         /* Create an empty File - before indexing to make sure can write to the index */
         sw->indexlist->DB = (void *) DB_Create(sw, sw->indexlist->line);
         if ( sw->lasterror )
             SwishAbortLastError( sw );
-
-        sw->Index->update_mode = 0;
     }
 
 
@@ -1403,10 +1440,13 @@ static void cmd_merge( SWISH *sw_input, CMDPARAMS *params )
 
     /* create output */
     sw_out = swish_new();
+    sw_out->verbose = sw_input->verbose;
+    sw_out->headerOutVerbose = sw_input->headerOutVerbose;
+
 
     addindexfile(sw_out, params->merge_out_file);
 
-        
+
     /* Update Economic mode */
     sw_out->Index->swap_locdata = params->swap_mode;
 
@@ -1419,6 +1459,7 @@ static void cmd_merge( SWISH *sw_input, CMDPARAMS *params )
 
     merge_indexes( sw_input, sw_out );
 
+
     write_index_file( sw_out, 0, elapsedStart, cpuStart, 1);
 
     swish_close( sw_out );
@@ -1437,7 +1478,7 @@ static void cmd_keywords( SWISH *sw, CMDPARAMS *params )
         addindexfile(sw, INDEXFILE);
 
     OutputKeyChar(sw, (int) (unsigned char) params->keychar);
-}    
+}
 
 
 /*************************************************************************
@@ -1458,12 +1499,12 @@ static void cmd_search( SWISH *sw, CMDPARAMS *params )
         addindexfile(sw, INDEXFILE);
 
 
-    /* Open index files */        
+    /* Open index files */
 
-    if ( !SwishAttach(sw) ) 
+    if ( !SwishAttach(sw) )
         SwishAbortLastError( sw );
 
-    
+
     srch = New_Search_Object( sw, params->query );
     if ( sw->lasterror )
         SwishAbortLastError( sw );
@@ -1471,6 +1512,8 @@ static void cmd_search( SWISH *sw, CMDPARAMS *params )
 
     srch->PhraseDelimiter = params->PhraseDelimiter;
     srch->structure       = params->structure;
+
+
     if ( params->sort_params )
     {
         srch->sort_params = params->sort_params;
@@ -1497,7 +1540,7 @@ static void cmd_search( SWISH *sw, CMDPARAMS *params )
 
         initSearchResultProperties(sw);
     }
-    
+
 
 
 
@@ -1529,7 +1572,7 @@ static void cmd_search( SWISH *sw, CMDPARAMS *params )
 
         resultHeaderOut(sw, 1, ".\n");
     }
-    else 
+    else
         resultHeaderOut(sw, 1, "err: no results\n.\n");
 
 
@@ -1555,7 +1598,7 @@ static void write_index_file( SWISH *sw, int process_stopwords, double elapsedSt
     int stopwords = 0;
     struct swline *cur_line;
 
-	/* Coalesce all remaining locations */
+        /* Coalesce all remaining locations */
     coalesce_all_word_locations(sw, sw->indexlist);
 
     if ( process_stopwords )
@@ -1622,7 +1665,7 @@ static void write_index_file( SWISH *sw, int process_stopwords, double elapsedSt
 
     fflush(stdout);
 
-    write_header(sw, &sw->indexlist->header, sw->indexlist->DB, sw->indexlist->line, sw->indexlist->header.totalwords, totalfiles, sw->indexlist->header.removedfiles, merge);
+    write_header(sw, &sw->indexlist->header, sw->indexlist->DB, sw->indexlist->line, sw->indexlist->header.totalwords, totalfiles, sw->indexlist->total_word_positions, sw->indexlist->header.removedfiles, merge);
 
     fflush(stdout);
 
@@ -1691,7 +1734,7 @@ static void display_result_headers( RESULTS_OBJECT *results )
     SWISH       *sw = results->sw;
     DB_RESULTS  *db_results = results->db_results;
 
-    
+
     resultHeaderOut(sw, 1, "%s\n", INDEXHEADER);  /* print # SWISH format: <version> */
 
     /* print out "original" search words */
@@ -1710,7 +1753,7 @@ static void display_result_headers( RESULTS_OBJECT *results )
 
         /* $$$ move to headers.c and fix as it's corrupting memory */
        // translatecharHeaderOut(sw, 2, &indexf->header);
-        
+
 
 
         /* Show parsed words */
