@@ -1,5 +1,5 @@
 /*
-$Id: swstring.c,v 1.1.2.1 2003/11/20 22:31:50 whmoseley Exp $
+$Id: swstring.c,v 1.3 2004/10/26 18:46:38 whmoseley Exp $
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
@@ -732,7 +732,7 @@ static const unsigned char iso8859_to_ascii7_lower_map[] = {
     '8', '9', ':', ';', '<', '=', '>', '?',
     '@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', /* 64 */
     'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
-    'p', 'q', 'r', 's', 't', 'u', 'v', 'q', /* 80 */
+    'p', 'q', 'r', 's', 't', 'u', 'v', 'w', /* 80 */
     'x', 'y', 'z', '[', '\\', ']', '^', '_',
     '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', /* 96 */
     'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
