@@ -18,10 +18,9 @@
 /* WORDDATA Block size */
 #define WORDDATA_BlockSize (WORDDATA_PageSize >> 8)
 
-#define SizeInt32 4
 
 /* Round to WORDDATA_PageSize */
-#define WORDDATA_RoundPageSize(n) (((n) + WORDDATA_PageSize - 1) & (~(WORDDATA_PageSize - 1)))
+#define WORDDATA_RoundPageSize(n) (sw_off_t)(((sw_off_t)(n) + (sw_off_t)(WORDDATA_PageSize - 1)) & (sw_off_t)(~(WORDDATA_PageSize - 1)))
 
 /* Round a number to the upper BlockSize */
 #define WORDDATA_RoundBlockSize(n) (((n) + WORDDATA_BlockSize - 1) & (~(WORDDATA_BlockSize - 1)))
@@ -33,28 +32,27 @@
 #define WORDDATA_MaxDataSize (WORDDATA_PageSize - WORDDATA_PageHeaderSize)
 
 #define WORDDATA_PageData(pg) ((pg)->data + WORDDATA_PageHeaderSize)
-#define WORDDATA_Data(pg,i) (WORDDATA_PageData((pg)) + (i) * SizeInt32)
 
-#define WORDDATA_SetBlocksInUse(pg,num) ( *(int *)((pg)->data + 0 * SizeInt32) = PACKLONG(num))
-#define WORDDATA_GetBlocksInUse(pg,num) ( (num) = UNPACKLONG(*(int *)((pg)->data + 0 * SizeInt32)))
+#define WORDDATA_SetBlocksInUse(pg,num) ( *(int *)((pg)->data + 0 * sizeof(int)) = PACKLONG(num))
+#define WORDDATA_GetBlocksInUse(pg,num) ( (num) = UNPACKLONG(*(int *)((pg)->data + 0 * sizeof(int))))
 
-#define WORDDATA_SetNumRecords(pg,num) ( *(int *)((pg)->data + 1 * SizeInt32) = PACKLONG(num))
-#define WORDDATA_GetNumRecords(pg,num) ( (num) = UNPACKLONG(*(int *)((pg)->data + 1 * SizeInt32)))
+#define WORDDATA_SetNumRecords(pg,num) ( *(int *)((pg)->data + 1 * sizeof(int)) = PACKLONG(num))
+#define WORDDATA_GetNumRecords(pg,num) ( (num) = UNPACKLONG(*(int *)((pg)->data + 1 * sizeof(int))))
 
 
 int WORDDATA_WritePageToDisk(FILE *fp, WORDDATA_Page *pg)
 {
     WORDDATA_SetBlocksInUse(pg,pg->used_blocks);
     WORDDATA_SetNumRecords(pg,pg->n);
-    fseek(fp,pg->page_number * WORDDATA_PageSize,SEEK_SET);
-    if ( fwrite(pg->data,WORDDATA_PageSize,1,fp) != 1 )
+    sw_fseek(fp,(sw_off_t)(pg->page_number * (sw_off_t)WORDDATA_PageSize),SEEK_SET);
+    if ( sw_fwrite(pg->data,WORDDATA_PageSize,1,fp) != 1 )
         progerrno("Failed to write page to disk: "); 
     return 1;
 }
 
 int WORDDATA_WritePage(WORDDATA *b, WORDDATA_Page *pg)
 {
-int hash = pg->page_number % WORDDATA_CACHE_SIZE;
+int hash = (int)(pg->page_number % (sw_off_t)WORDDATA_CACHE_SIZE);
 WORDDATA_Page *tmp;
     pg->modified =1;
     if((tmp = b->cache[hash]))
@@ -120,12 +118,12 @@ WORDDATA_Page *tmp,*next;
     return 0;
 }
 
-WORDDATA_Page *WORDDATA_ReadPageFromDisk(FILE *fp, unsigned long page_number)
+WORDDATA_Page *WORDDATA_ReadPageFromDisk(FILE *fp, sw_off_t page_number)
 {
 WORDDATA_Page *pg = (WORDDATA_Page *)emalloc(sizeof(WORDDATA_Page) + WORDDATA_PageSize);
 
-    fseek(fp,page_number * WORDDATA_PageSize,SEEK_SET);
-    fread(pg->data,WORDDATA_PageSize, 1, fp);
+    sw_fseek(fp,(sw_off_t)(page_number * (sw_off_t)WORDDATA_PageSize),SEEK_SET);
+    sw_fread(pg->data,WORDDATA_PageSize, 1, fp);
 
     WORDDATA_GetBlocksInUse(pg,pg->used_blocks);
     WORDDATA_GetNumRecords(pg,pg->n);
@@ -135,9 +133,9 @@ WORDDATA_Page *pg = (WORDDATA_Page *)emalloc(sizeof(WORDDATA_Page) + WORDDATA_Pa
     return pg;
 }
 
-WORDDATA_Page *WORDDATA_ReadPage(WORDDATA *b, unsigned long page_number)
+WORDDATA_Page *WORDDATA_ReadPage(WORDDATA *b, sw_off_t page_number)
 {
-int hash = page_number % WORDDATA_CACHE_SIZE;
+int hash = (int)(page_number % (sw_off_t)WORDDATA_CACHE_SIZE);
 WORDDATA_Page *tmp;
     if((tmp = b->cache[hash]))
     {
@@ -163,19 +161,19 @@ WORDDATA_Page *tmp;
 WORDDATA_Page *WORDDATA_NewPage(WORDDATA *b)
 {
 WORDDATA_Page *pg;
-long offset;
+sw_off_t offset;
 FILE *fp = b->fp;
 int hash;
-int size = WORDDATA_PageSize;
 int i;
-unsigned long page_number =0;
+sw_off_t page_number = (sw_off_t)0;
+unsigned char empty_buffer[WORDDATA_PageSize];
     /* Let's see if we have a previous available page */
     if(b->num_Reusable_Pages)
     {
         /* First, look for a page of the same size */
         for(i = 0; i < b->num_Reusable_Pages ; i++)
         {
-            if(size == b->Reusable_Pages[i].page_size)
+            if(WORDDATA_PageSize == b->Reusable_Pages[i].page_size)
                 break;
         }
         /* If not found, let's try with a bigger one if exits */
@@ -183,7 +181,7 @@ unsigned long page_number =0;
         {
             for(i = 0; i < b->num_Reusable_Pages ; i++)
             {
-                if(size < b->Reusable_Pages[i].page_size)
+                if(WORDDATA_PageSize < b->Reusable_Pages[i].page_size)
                     break;
             }
         }
@@ -191,7 +189,7 @@ unsigned long page_number =0;
         if(i != b->num_Reusable_Pages)
         {
             page_number = b->Reusable_Pages[i].page_number;
-            if(size == b->Reusable_Pages[i].page_size)
+            if(WORDDATA_PageSize == b->Reusable_Pages[i].page_size)
             {
                 for(++i;i<b->num_Reusable_Pages;i++)
                 {
@@ -203,8 +201,8 @@ unsigned long page_number =0;
             }
             else
             {
-                b->Reusable_Pages[i].page_number += size/WORDDATA_PageSize;
-                b->Reusable_Pages[i].page_size -= size;
+                b->Reusable_Pages[i].page_number ++;
+                b->Reusable_Pages[i].page_size -= WORDDATA_PageSize;
             }
         }
     }
@@ -212,26 +210,29 @@ unsigned long page_number =0;
     if(! page_number)
     {
         /* Get file pointer */
-        if(fseek(fp,0,SEEK_END) !=0)
+        if(sw_fseek(fp,(sw_off_t)0,SEEK_END) !=0)
             progerrno("Internal error seeking: "); 
 
 
-        offset = ftell(fp);
+        offset = sw_ftell(fp);
         /* Round up file pointer */
         offset = WORDDATA_RoundPageSize(offset);
 
         /* Set new file pointer - data will be aligned */
-        if(fseek(fp,offset, SEEK_SET)!=0 || offset != ftell(fp))
+        if(sw_fseek(fp,offset, SEEK_SET)!=0 || offset != sw_ftell(fp))
             progerrno("Internal error seeking: "); 
 
-        if(fwrite("\0",1,size,fp)!=size || ((long)size + offset) != ftell(fp))
+        /* Reserve space in file */
+        memset(empty_buffer,'0',WORDDATA_PageSize);
+
+        if(sw_fwrite(empty_buffer,1,WORDDATA_PageSize,fp)!=WORDDATA_PageSize || ((sw_off_t)WORDDATA_PageSize + offset) != sw_ftell(fp))
             progerrno("Faild to write page data: ");
 
-        page_number = offset/WORDDATA_PageSize;
+        page_number = offset / (sw_off_t)WORDDATA_PageSize;
     }
 
-    pg = (WORDDATA_Page *)emalloc(sizeof(WORDDATA_Page) + size);
-    memset(pg,0,sizeof(WORDDATA_Page) + size);
+    pg = (WORDDATA_Page *)emalloc(sizeof(WORDDATA_Page) + WORDDATA_PageSize);
+    memset(pg,0,sizeof(WORDDATA_Page) + WORDDATA_PageSize);
         /* Reserve space in file */
 
     pg->used_blocks = 0;
@@ -242,7 +243,7 @@ unsigned long page_number =0;
     /* add to cache */
     pg->modified = 1;
     pg->in_use = 1;
-    hash = pg->page_number % WORDDATA_CACHE_SIZE;
+    hash = (int)(pg->page_number % (sw_off_t)WORDDATA_CACHE_SIZE);
     pg->next_cache = b->cache[hash];
     b->cache[hash] = pg;
     return pg;
@@ -250,7 +251,8 @@ unsigned long page_number =0;
 
 void WORDDATA_FreePage(WORDDATA *b, WORDDATA_Page *pg)
 {
-int hash = pg->page_number % WORDDATA_CACHE_SIZE;
+int hash = (int)(pg->page_number % (sw_off_t)WORDDATA_CACHE_SIZE);
+
 WORDDATA_Page *tmp;
 
     tmp = b->cache[hash];
@@ -270,28 +272,16 @@ WORDDATA_Page *tmp;
 WORDDATA *WORDDATA_New(FILE *fp)
 {
 WORDDATA *b;
-int i;
     b = (WORDDATA *) emalloc(sizeof(WORDDATA));
+    memset(b,0,sizeof(WORDDATA));
     b->fp = fp;
-    b->last_put_page = 0;
-    b->last_del_page = 0;
-    b->last_get_page = 0;
-    b->page_counter = 0;
-    for(i = 0; i < WORDDATA_CACHE_SIZE; i++)
-        b->cache[i] = NULL;
-
-    b->num_Reusable_Pages = 0;
     return b;
 }
 
 
 WORDDATA *WORDDATA_Open(FILE *fp)
 {
-WORDDATA *b;
-
-    b = WORDDATA_New(fp);
-
-    return b;
+    return WORDDATA_New(fp);
 }
 
 void WORDDATA_Close(WORDDATA *bt)
@@ -302,14 +292,14 @@ void WORDDATA_Close(WORDDATA *bt)
 }
 
 
-unsigned long WORDDATA_PutBig(WORDDATA *b, unsigned int len, unsigned char *data)
+sw_off_t WORDDATA_PutBig(WORDDATA *b, unsigned int len, unsigned char *data)
 {
-long offset;
-int size = WORDDATA_RoundPageSize(sizeof(unsigned long) + len);
+sw_off_t offset;
 unsigned long p_len = (unsigned long)PACKLONG((unsigned long)len);
+int size = WORDDATA_RoundPageSize(sizeof(p_len) + len);
 FILE *fp = b->fp;
-unsigned long id;
-unsigned long page_number = 0;
+sw_off_t id;
+sw_off_t page_number = (sw_off_t)0;
 int i;
     /* Let's see if we have a previous available page */
     if(b->num_Reusable_Pages)
@@ -353,34 +343,34 @@ int i;
     if(! page_number)
     {
         /* Get file pointer */
-        if(fseek(fp,0,SEEK_END) !=0)
+        if(sw_fseek(fp,(sw_off_t)0,SEEK_END) !=0)
             progerrno("Internal error seeking: "); 
 
-        offset = ftell(fp);
+        offset = sw_ftell(fp);
         /* Round up file pointer */
         offset = WORDDATA_RoundPageSize(offset);
     }
     else
     {
-        offset = page_number * WORDDATA_PageSize;
+        offset = page_number * (sw_off_t)WORDDATA_PageSize;
     }
     /* Set new file pointer - data will be aligned */
-    if(fseek(fp,offset, SEEK_SET)!=0 || offset != ftell(fp))
+    if(sw_fseek(fp,offset, SEEK_SET)!=0 || offset != sw_ftell(fp))
         progerrno("Internal error seeking: "); 
 
-    id = ((unsigned long) (offset / WORDDATA_PageSize)) << 8;
+    id = (sw_off_t)(((offset / (sw_off_t)WORDDATA_PageSize)) << (sw_off_t)8);
 
     /* Write packed length */
-    fwrite(&p_len,1,SizeInt32,fp);
+    sw_fwrite(&p_len,1,sizeof(p_len),fp);
     /* Write data */
-    fwrite(data,1,len,fp);
+    sw_fwrite(data,1,len,fp);
 
     /* New offset */
-    offset = ftell(fp);
+    offset = sw_ftell(fp);
     /* Round up file pointer */
     offset = WORDDATA_RoundPageSize(offset);
     /* Set new file pointer - data will be aligned */
-    if(fseek(fp,offset, SEEK_SET)!=0 || offset != ftell(fp))
+    if(sw_fseek(fp,offset, SEEK_SET)!=0 || offset != sw_ftell(fp))
         progerrno("Internal error seeking: "); 
 
     b->lastid = id;
@@ -388,7 +378,7 @@ int i;
 }
 
 
-unsigned long WORDDATA_Put(WORDDATA *b, unsigned int len, unsigned char *data)
+sw_off_t WORDDATA_Put(WORDDATA *b, unsigned int len, unsigned char *data)
 {
 int required_length;
 int free_blocks;
@@ -494,26 +484,26 @@ WORDDATA_Page *last_page=NULL;
     last_page->n++;
     last_page->used_blocks += required_length / WORDDATA_BlockSize;
     WORDDATA_WritePage(b,last_page);
-    return (unsigned long)(b->lastid=(unsigned long)((last_page->page_number << 8) + free_id));
+    return (b->lastid=(sw_off_t)((sw_off_t)(last_page->page_number << (sw_off_t)8) + (sw_off_t)free_id));
 }
 
-unsigned char *WORDDATA_GetBig(WORDDATA *b, unsigned long page_number, unsigned int *len)
+unsigned char *WORDDATA_GetBig(WORDDATA *b, sw_off_t page_number, unsigned int *len)
 {
-long offset = (long) (page_number * WORDDATA_PageSize);
+sw_off_t offset = page_number * (sw_off_t)WORDDATA_PageSize;
 unsigned long p_len;
 unsigned char *data;
-    fseek(b->fp, offset, SEEK_SET);
-    fread(&p_len,1,SizeInt32,b->fp);
+    sw_fseek(b->fp, offset, SEEK_SET);
+    sw_fread(&p_len,1,sizeof(p_len),b->fp);
     *len = UNPACKLONG(p_len);
     data = (unsigned char *)emalloc(*len);
-    fread(data,1,*len,b->fp);
+    sw_fread(data,1,*len,b->fp);
     return data;
 }
 
-unsigned char *WORDDATA_Get(WORDDATA *b, unsigned long global_id, unsigned int *len)
+unsigned char *WORDDATA_Get(WORDDATA *b, sw_off_t global_id, unsigned int *len)
 {
-unsigned long page_number = global_id >> 8;
-int id = (int)(global_id & 0xff);
+sw_off_t page_number = global_id >> (sw_off_t)8;
+int id = (int)(global_id & (sw_off_t)0xff);
 int r_id=-1,r_len=-1;
 int i;
 unsigned char *p;
@@ -554,13 +544,13 @@ unsigned char *data;
     return data;
 }
 
-void WORDDATA_DelBig(WORDDATA *b, unsigned long page_number, unsigned int *len)
+void WORDDATA_DelBig(WORDDATA *b, sw_off_t page_number, unsigned int *len)
 {
-long offset = (long) (page_number * WORDDATA_PageSize);
+sw_off_t offset = page_number * (sw_off_t)WORDDATA_PageSize;
 unsigned long p_len;
-    fseek(b->fp, offset, SEEK_SET);
-    fread(&p_len,1,SizeInt32,b->fp);
-    *len = UNPACKLONG(p_len) + sizeof(unsigned long);
+    sw_fseek(b->fp, offset, SEEK_SET);
+    sw_fread(&p_len,1,sizeof(p_len),b->fp);
+    *len = UNPACKLONG(p_len) + sizeof(p_len);
 
     if(b->num_Reusable_Pages < WORDDATA_MAX_REUSABLE_PAGES)
     {
@@ -569,10 +559,10 @@ unsigned long p_len;
     }
 }
 
-void WORDDATA_Del(WORDDATA *b, unsigned long global_id, unsigned int *len)
+void WORDDATA_Del(WORDDATA *b, sw_off_t global_id, unsigned int *len)
 {
-unsigned long page_number = global_id >> 8;
-int id = (int)(global_id & 0xff);
+sw_off_t page_number = global_id >> (sw_off_t)8;
+int id = (int)(global_id & (sw_off_t)0xff);
 int r_id=-1,r_len=-1,tmp;
 int i;
 unsigned char *p, *q;
@@ -671,11 +661,11 @@ static unsigned long nums[N_TEST];
 
 
 
-    fp = fopen("kkkkk",FILEMODE_WRITE);
-    fclose(fp);
-    fp = fopen("kkkkk",FILEMODE_READWRITE);
+    fp = sw_fopen("kkkkk",FILEMODE_WRITE);
+    sw_fclose(fp);
+    fp = sw_fopen("kkkkk",FILEMODE_READWRITE);
 
-    fwrite("aaa",1,3,fp);
+    sw_fwrite("aaa",1,3,fp);
 
 printf("\n\nIndexing\n\n");
 
@@ -693,12 +683,12 @@ printf("\n\nIndexing\n\n");
     }
 
     WORDDATA_Close(bt);
-    fclose(fp);
+    sw_fclose(fp);
 
 printf("\n\nUnfreed %d\n\n",num);
 printf("\n\nSearching\n\n");
 
-    fp = fopen("kkkkk",FILEMODE_READ);
+    fp = sw_fopen("kkkkk",FILEMODE_READ);
     bt = WORDDATA_Open(fp);
 
     for(i=0;i<N_TEST;i++)
@@ -711,7 +701,7 @@ printf("\n\nSearching\n\n");
 
     WORDDATA_Close(bt);
 
-    fclose(fp);
+    sw_fclose(fp);
 printf("\n\nUnfreed %d\n\n",num);
 
 }
